#!/usr/bin/env python
"""
This script tries to install lots of Python modules.
Most of them install trivially by a python setup.py install,
but SciPy and PySparse may cause problems.
All the simple installs are run before SciPy and PySparse, so
if this script aborts, you probably have most of the modules.
Try to issue the installation commands manually for all packages
that will not install automatically.
"""

import os, sys, re, shutil, glob
from subst import subst

#debug:
def echo(*args):
    for i in args:
        print i,
    print

#os.system = echo
#shutil.copy = echo
#os.rename = echo

def system(command, abort=True):
    """Run os.system and allow to abort if failure."""
    failure = os.system(command)
    if failure:
        msg = 'Could not run system command\n  "%s"\n' % command
        #print msg
        if abort:
            raise OSError, msg
            #sys.exit(1)

    
sysdir = os.environ['SYSDIR']
prefix = os.environ['PREFIX']
d = os.environ['SYSDIR'] + '/' + os.environ['MACHINE_TYPE']
if not prefix == d:
    raise TypeError, "Environment variable PREFIX is not defined as %s" % d
mmprefix = '--prefix='+os.environ['PREFIX']
pyver = sys.version[0:3]

home_install = os.path.join(os.path.join(os.environ['HOME'], 'install',
                                         'lib', 'python'))
site_packages_install = '%s/lib/python%s/site-packages' % (prefix, pyver)

# Does it makes sense to install under site_packages_install, i.e.,
# is Python installed there? If so, we use that path, otherwise we
# use a home directory path

if os.path.isdir(site_packages_install):
    setup_install_part = ''
else:
    setup_install_part = ' --home=%s/install' % os.environ['HOME']
    # if another installation directory for Python packages is desired,
    # just define
    #setup_install_part = ' --prefix='/my/favorite/install/path'

py_setup_install = 'python setup.py install ' + setup_install_part


def add_sitecustomize(path):
    """
    Add a path to the sys.path set in the sitecustomize module
    (sitecustomize is always imported by the site module, which
    is always imported at start-up of python).
    """
    # only necessary for a home_install
    sitefile = os.path.join(home_install, 'sitecustomize.py')
    if not os.path.isfile(sitefile):
        # "touch" the file:
        f = open(sitefile, 'w')
        f.write("import sys\n")
        f.close()
    # check if the path is already imported:
    if not path in sys.path:
        f = open(sitefile, 'a')
        f.write("sys.path.append('%s')\n" % path)
        f.close()
        sys.path.append(path) # be ready for forthcoming installs in this script
    

def isdir(dir):
    """
    Check if dir exists. If not, find some alternatives
    by removing all info about (version) numbers in the dir names.
    """
    if not os.path.isdir(dir):
        parent = os.path.dirname(dir)
        files = os.listdir(parent)
        dir2 = re.sub(r"\d","",dir)  # remove digits
        candidates = []
        for f in files:
            if os.path.isdir(f):
                f2 = re.sub(r"\d","",dir)
                if dir2 == f2:
                    candidates.append(f)
        if candidates:
            raise IOError, "could not find directory %s, but some alternatives are %s" % (dir, str(candidates))
        else:
            raise IOError, "could not find directory %s" % (dir)
    else:
        return 1
            
def install(root, dir, func):
    cwd = os.getcwd()
    d = os.path.join(root,dir)
    print d
    isdir(d)  # check that d is a directory
    os.chdir(d)
    func()
    os.chdir(cwd)


def disallow_home_scheme():
    # extension modules may have problems with the book set-up of
    # compilation if Numeric and other modules are not installed
    # along with Python itself, so do not allow the --home scheme:
    if setup_install_part:
        print 'Numeric should not be installed with --home but in %s instead' %  site_packages_install
        sys.exit(1)
    
def Numeric():
    disallow_home_scheme()
    
    print "Numeric dir:", os.getcwd()
    system("sh ./makeclean.sh")
    # alternative: import shutil; shutil.rmtree('build')

    # important fix for gcc and Numeric: CFLAGS='-ffloat-store' python setup.py
    system("CFLAGS='-ffloat-store' " + py_setup_install)
    system(py_setup_install)

    # Numeric.pth only works when Numeric is installed in sys.prefix/...
    # otherwise we need to add a sitecustomize module:
    # (but we do not recommend Numeric in other places)
    #add_sitecustomize(os.path.join(site_packages_install, 'Numeric'))

    # make libnumpy.so library file from _numpy.so:
    numpy_root    = os.path.join(site_packages_install, 'Numeric')
    oldname = numpy_root + '/_numpy.so'
    newname = numpy_root + '/libnumpy.so'
    if os.path.islink(newname): os.remove(newname)
    system("ln -s " + oldname + " " + newname)

def numarray():
    disallow_home_scheme()
    print "numarray dir:", os.getcwd()
    system("python setup.py install --gencode %s" % setup_install_part)


def Pmw():
    shutil.copy('contrib/PmwFileDialog.py', sysdir+'/src/python/lib')
    # we do not install Pmw, instead we place the parent directory
    # in PYTHONPATH, check that sys.path contains this dir
    os.chdir('../..')
    # check that Pmw and BLT play together:
    import Pmw
    try:
        s = Pmw.Blt.Graph
    except:
        print "WARNING: Python is not linked correctly with BLT"
        print "=> Pmw's BLT graph widget will not work"



def matplotlib():
    if not os.path.isfile('setupext.py-orig'):
        # make a copy of original file the first time
        shutil.copy('setupext.py', 'setupext.py-orig')
    shutil.copy('setupext.py-orig', 'setupext.py')
    # edit setupext.py: (add installations under $PREFIX)
    if 'PREFIX' in os.environ:
        addline = """

import os, sys
basedir[sys.platform].insert(0, os.environ['PREFIX'])

"""
        subst('import sys, os, stat', addline + 'import stat', 'setupext.py')

    # need NumPy arrayobject.h file, check local install:
    local_include = os.path.join(os.environ['HOME'],
                                 'install', 'include', 'python')
    if os.path.isdir(local_include):
        ### NOTE: this is not enough! see setupext.py-hpl
        liner = r'def add_base_flags\(module\):'
        linet = 'def add_base_flags(module):'
        addline = """
    module.include_dirs.append(os.path.join(os.environ['HOME'],
                               'install', 'include', 'python'))
"""
        subst(liner, linet + addline, 'setupext.py')

    # turn off Gtk (requires too much usually...):
    # ...
    system(py_setup_install)
    # set in .bashrc: MATPLOTLIBDATA
    #export MATPLOTLIBDATA=$HOME/install/share/matplotlib:$MATPLOTLIBDATA


def std():
    system(py_setup_install)

def PyX():
    system(py_setup_install + ' --root=$HOME/install')

def SWIG():
    system('make distclean')
    system('./configure --with-python='+prefix+'/bin/python ' + mmprefix)
    system('make; make install; make clean')

def Scientific():
    system(py_setup_install)
    # no netCDF

def scipy_core():
    disallow_home_scheme()
    os.chdir('scipy_core/scipy_distutils/')
    system(py_setup_install)
    os.chdir('../..')

def scipy_0_3():
    #not necessary: disallow_home_scheme()
    # must have compiled atlas (src/install/atlas)

    # make sure to use gcc/g77-3.4 and not v4.0

    #atlas_ver = 'Linux_PIIISSE1'  # for my Linux PC...
    #os.environ['ATLAS'] = \
    # os.path.join(sysdir, 'src', 'ATLAS', 'lib', atlas_ver)

    atlas = True
    if not 'ATLAS' in os.environ:
        atlas = False
    elif not os.path.isdir(os.environ['ATLAS']):
        atlas = False
    if not atlas:
        print """
Cannot install scipy because ATLAS is not installed. Follow
these instructions (Bourne shell syntax, place commands in a script):

# fetch http://www.netlib.org/lapack/lapack.tgz
# tar xvzf lapack.tgz
cd LAPACK
cp INSTALL/make.inc.LINUX make.inc
# remove prefix in library name:
subst.py 'PLAT = _LINUX' 'PLAT =' make.inc
cd BLAS/SRC; PATH=$PATH:.; make
cd ../..
PATH=$PATH:.; make lapacklib
cd ..

# fetch http://heanet.dl.sourceforge.net/sourceforge/math-atlas/atlas3.5.13.tar.bz2
# bunzip2 atlas3.5.13.tar.bz2
# tar xf atlas3.5.13.tar
cd ATLAS
hardware=Linux_PIIISSE1   # depends on the PC you have!!
make 
make install arch=$hardware

# make optimized LAPACK libraries:
cd lib/$hardware
mkdir tmp; cd tmp
ar x ../liblapack.a
cp ../../../../LAPACK/lapack.a ../liblapack.a
ar r ../liblapack.a *.o
cd ..; rm -rf tmp

# set the environment variable ATLAS to point to the
# directory with the ATLAS lib*.a files
"""
        sys.exit(1)
    # SciPy 0.3 does not work well with gcc-4.0
    build = 'python setup.py build --compiler=gcc-3.4'
    #system(build)  # does not work
    # alternative hpl hack:
    os.system('ln -s /usr/bin/gcc-3.4 ~/bin/gcc')  # ~/bin is before /usr/bin in PATH!!
    system(py_setup_install)
    os.system('rm -f ~/bin/gcc')  # remove link

def scipy():
    # latest version
    # use gcc/g77-3.4, not v4.0
    disallow_home_scheme()
    system(py_setup_install)
    
def pysparse():
    if not 'ATLAS' in os.environ:
        print 'ATLAS environment variable is not set; cannot install PySparse'
        print 'run install/lapack; install/atlas (manually)'
        print 'and try install/python_modules.py again'
        sys.exit(1)

    # insert ATLAS/LAPACK library specification:
    if not os.path.isfile('setup.py-local'):
        shutil.copy('setup.py', 'setup.py-local')
        newtext = """

# link to ATLAS version of BLAS and LAPACK:
library_dirs_list = [os.path.join(os.environ['ATLAS'])]
libraries_list = ['lapack', 'f77blas', 'cblas', 'atlas', 'g2c']

"""
        subst('ext_modules = ', newtext + 'ext_modules = ', 'setup.py-local')
    system(py_setup_install)
    
def scxx():
    system('./make.sh')
    system('./clean.sh')

def rpy():
    os.environ['RHOME'] = os.path.join('/usr','lib','R')
    if not os.path.islink('%s/lib/libR.so' % prefix):
        system('ln -s %s/bin/libR.so %s/lib/libR.so' % \
                  (os.environ['RHOME'],prefix))
    system(py_setup_install)

def Imaging():
    try: system(py_setup_install)
    except OSError, e:
        print "Failed building Imaging", e
        print "\nTrying build without tk support\n============================"
        setup = open('setup.py').read()
        f = open('setup_notk.py', 'w')
        f.write(setup.replace(r"""
try:
    import _tkinter
except ImportError:
    _tkinter = None""",
"""\n_tkinter = None""", 1)	    
        )
        f.close()
        system(py_setup_install.replace('setup.py', 'setup_notk.py'))

root = sysdir + '/src/python/tools'

software_list = [
    ('Numeric', 'Numeric-24.2', Numeric),
    ('numarray', 'numarray-1.5.1', numarray),
    ('numpy', 'numpy-0.9.6', std),
    ('Gnuplot', 'gnuplot-py-1.7', std),
    ('IPython', 'ipython-0.6.15', std),
    ('PIL', 'Imaging-1.1.5', Imaging),
    ('Pmw', 'Pmw/Pmw_1_2', Pmw),
    ('PmwContribD', 'PmwContribD-r2_0_2', std),
    #('pymat', 'pymat-new', std)  # from CVS,
    ('TableIO', 'TableIO', std),
    ('Docutils', 'docutils-0.3.9', std),
    ('ScientificPython', 'ScientificPython-2.4.9', Scientific),
    #('HappyDoc', 'HappyDoc-r3_0_a1', std)  # got problems...,
    ('HappyDoc', 'HappyDoc-r2_1', std),
    ('SCXX', 'scxx', scxx),
    ('Unum', 'Unum-4.0', std),
    ('pylint', 'pylint-0.3.3', std),
    ('preprocess', 'preprocess-1.0.0', std),
    ('CNRG', 'crng-1.2', std),
    ('Epydoc', 'epydoc-2.1', std),
    #('Matplotlib', 'matplotlib-0.86.2', matplotlib),
    ('Psyco', 'psyco-1.5', std),
    ('Pexpect', 'pexpect-2.1beta', std),
    ('PyX', 'PyX-0.8.1', PyX),
    ('PyXML', 'PyXML-0.8.4', std),
    ('Gnosis', 'Gnosis_Utils-1.2.0', std),
    ('SWIG', 'swig-1.3.27', SWIG),
    #('RPy', 'rpy-0.3.1', rpy),

    # need to have installed lapack and atlas,
    # see install/lapack, install/atlas, before pysparse and scipy!

    # --- SciPy ---
    # note that SciPy breaks with gcc 4.0, use gcc 3.4
    # safe and old SciPy:
    #('SciPy-core', 'SciPy_complete-0.3.2', scipy_core),
    ('SciPy', 'SciPy_complete-0.3.2', scipy_0_3),
    # new SciPy:
    #('SciPy', scipy-0.4.8, std),

    # need to physical remove old f2py2e and bin/f2py to install new one
    # no, we just use the new one that comes with numpy
    #('F2PY', 'F2PY-2.45.241_1926', std),

    ('PySparse', 'pysparse-0.34.032', pysparse),  # each platform needs special coding in setup.py
    ]

# make a dictionary version of software_list (the bad thing with a
# dictionary is that we have no sequence of the installation, but
# we can use the dictionary to install just a few packages)
software_dict = {}
for name, directory, func in software_list:
    software_dict[name] = (directory, func)
    
def install_all():
    for name, directory, func in software_list:
        install(root, directory, func)

if __name__ == '__main__':
    if len(sys.argv) < 2:
        install_all()
    else:
        for package in sys.argv[1:]:
            if package in software_dict:
                install(root, software_dict[package][0],
                        software_dict[package][1])
            else:
                print '%s is not a legal name. '\
                      'Here is the list of packages:\n%s' % \
                      (package, str(software_dict.keys())[1:-1])
                sys.exit(1)
                      
            


"""
# libplot and biggles require GNU's plotutils
# (compilation may fail if you don't have it)

libplot="python-libplot-1.0.2"
if [ -d $libplot ]; then
  # requires compile of plotutils:
  if [ -x $SYSDIR/src/install/plotutils ]; then
    cd $SYSDIR/src
  
    install/plotutils
    cd python/tools

    cd $libplot
    make install
    make clean
    cd ..
  fi

# make sure that Makefile has -I and -L to the SYSDIR dirs:
#CPPFLAGS	= -I$(pyprefix)/include/python$(pyversion) \
#                  -I/usr/local/include -I$(SYSDIR)/$(MACHINE_TYPE)/include
#LDFLAGS		= -L/usr/local/lib -lplot -L/usr/X11R6/lib \
#                  -L$(SYSDIR)/$(MACHINE_TYPE)/lib \
#		  -lXaw -lXmu -lXt -lSM -lICE -lXext -lX11 -lm
fi

biggles="python-biggles-1.3.0"
if [ -d $biggles ]; then
  if [ -d $SYSDIR/src/install/plotutils ]; then
    cd $biggles
    make install
    cd ..
  fi
fi
"""

