#!/usr/bin/env python
#
# $Id: DelayedCreatePage.py,v 1.3 2001/11/03 11:05:22 doughellmann Exp $
#
# Copyright 2001 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Subclassable version of the page for a Pmw.NoteBookR.

  **This appears to be obsolete in Pmw 0.8.5 because NoteBookR is not
  there.**

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: DelayedCreatePage.py,v $',
    'rcs_id'       : '$Id: DelayedCreatePage.py,v 1.3 2001/11/03 11:05:22 doughellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'PmwContribD',
    'created'      : 'Sun, 01-Apr-2001 13:00:14 EDT',

    #
    #  Current Information
    #
    'author'       : '$Author: doughellmann $',
    'version'      : '$Revision: 1.3 $',
    'date'         : '$Date: 2001/11/03 11:05:22 $',
}

#
# Import system modules
#
import Pmw
# this is required to force Pmw to be initialized
_ignoreMe = Pmw.NoteBook
# now tell Pmw where to find the NoteBookPage
# this is a hack, since the class is not exported
# we have to do this to subclass it
Pmw._widgets['NoteBookPage'] = Pmw._widgets['NoteBook']
Pmw._widgets_keys.append('NoteBookPage')

#
# Import Local modules
#


#
# Module
#




class DelayedCreatePage(Pmw.NoteBookPage):
    """A notebook page which does not create its contents lifted.

        Options

          'label' -- Lable to appear on the page tab.

          'createcmd' -- Function to be called to create the contents
          of this page.

          'createnow' -- Boolean indicating whether or not to create
          the contents right away (sometimes this is desirable).
          Defaults to '0'.
          
    """

    title = 'Set the title class attribute'

    def __init__(self, notebook, pagename, **kw):
        """Create the DelayedCreatePage.

        Instantiate the in-memory, but not on-screen, representation
        of the widget.

        Arguments

          'notebook' -- Notebook parent widget.

          'pagename' -- Name of this page being created.

        """
        INITOPT = Pmw.INITOPT
        optiondefs = (
            ('label',     self.title,          INITOPT),
            ('createcmd', self.createcontents, INITOPT),
            ('createnow', 0,                   INITOPT),
        )
        self.defineoptions(kw, optiondefs)
        Pmw.NoteBookPage.__init__(self, notebook, pagename)
        if self['createnow']:
            self.createcontents()
            self._iscreated = 1
        self.initialiseoptions(self.__class__)
        return

    def createcontents(self):
        """Called when lifted to create the contained widgets.

        Do not override this method.  Instead, specify a
        'createcmd' value when the widget is instantiated.
        """
        if self._iscreated:
            return
        self._iscreated = 1
        self.createinside()
        return
