#!/usr/bin/env python
#
# $Id: DisplayField.py,v 1.3 2001/11/03 11:05:22 doughellmann Exp $
#
# Copyright 2001 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Widget to display a value with a label.

  This should behave similarly to the Pmw.EntryField, but not allow
  editing and hopefully take up fewer resources.

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: DisplayField.py,v $',
    'rcs_id'       : '$Id: DisplayField.py,v 1.3 2001/11/03 11:05:22 doughellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'PmwContribD',
    'created'      : 'Sun, 01-Apr-2001 13:01:09 EDT',

    #
    #  Current Information
    #
    'author'       : '$Author: doughellmann $',
    'version'      : '$Revision: 1.3 $',
    'date'         : '$Date: 2001/11/03 11:05:22 $',
}

#
# Import system modules
#
import Tkinter
import Pmw


#
# Import Local modules
#


#
# Module
#

class DisplayField(Pmw.LabeledWidget):
    """Widget to display a value with a label.

      This should behave similarly to the Pmw.EntryField, but not
      allow editing and hopefully take up fewer resources.

    Components
    
        label -- the label of the data
        
        entry -- the label with the data

    Options

        labelpos -- Position of the label widget.  Defaults to 'w'.

        label_anchor -- Justification or anchoring of label text.
        Defaults to 'w'.
        
    """
    
    def __init__(self, parent, **kw):
        INITOPT = Pmw.INITOPT
        optiondefs = (
            ('labelpos', 'w', INITOPT),
            ('label_anchor', 'w', INITOPT),
        )
        self.defineoptions(kw, optiondefs)
        Pmw.LabeledWidget.__init__(self, parent)
        w = self.createcomponent(
            'entry',
            (), None,
            Tkinter.Label,
            (self.interior(),),
            relief=Tkinter.GROOVE,
            anchor='w',
            )
        w.pack(
            side=Tkinter.TOP,
            expand=Tkinter.YES,
            fill=Tkinter.BOTH,
            )
        self.component('label').grid(sticky='nsew')
        self.initialiseoptions(self.__class__)
        return
    
if __name__ == '__main__':
    from GuiAppD import GuiAppD
    class TestGui(GuiAppD):
        def createInterface(self):
            var = Tkinter.StringVar(self.interior())
            var.set('Here is the value')
            field = self.createcomponent(
                'field',
                (), None,
                DisplayField,
                (self.interior(),),
                entry_textvariable=var,
                label_text='This is the label',
                )
            field.pack()
            return
    TestGui().run()

