#!/usr/bin/env python
#
# $Id: wcc_quickwacky.py,v 1.3 2001/11/03 11:05:22 doughellmann Exp $
#
# Copyright 2001 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Privide a command line interface to send a quick wacky message
through a WackyChat server.


"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: wcc_quickwacky.py,v $',
    'rcs_id'       : '$Id: wcc_quickwacky.py,v 1.3 2001/11/03 11:05:22 doughellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'UNSPECIFIED',
    'created'      : 'Sun, 27-May-2001 12:00:30 EDT',

    #
    #  Current Information
    #
    'author'       : '$Author: doughellmann $',
    'version'      : '$Revision: 1.3 $',
    'date'         : '$Date: 2001/11/03 11:05:22 $',
}

#
# Import system modules
#
import os
import string


#
# Import Local modules
#
import wcc_protocol


#
# Module
#

from CommandLineApp import CommandLineApp

class QuickWacky(CommandLineApp):

    def appInit(self):
        try:
            self.uid = os.environ['WACKY_USER']
        except KeyError:
            self.uid = 0
        try:
            self.password = os.environ['WACKY_PASSWORD']
        except KeyError:
            self.password = ''
        try:
            self.server_name = os.environ['WACKY_SERVER']
        except KeyError:
            self.server_name = 'localhost'
        try:
            self.port = os.environ['WACKY_PORT']
        except KeyError:
            self.port = 2980
        self.to_address = []
        return

    
    def optionHandler_u(self, uid):
        """
        Specify the Wacky user id.  The default can be specified
        with the environment variable WACKY_USER.
        """
        self.uid = uid
        return

    def optionHandler_p(self, password):
        """
        Specify the password for the Wacky user id.  The default
        can be specified with the environment variable WACKY_PASSWORD.
        """
        self.password = password
        return

    def optionHandler_t(self, to_user):
        """
        Specify the uid to whom the message should be sent.  This
        option can be repeated to send a message to more than one
        person.
        """
        self.to_address.append(to_user)
        return
    
    def main(self, *args):
        try:
            self.server = wcc_protocol.WackyConnection(
                hostname=self.server_name,
                port=self.port,
                userid=self.uid,
                password=self.password,
                )
        except wcc_protocol.WackyProtocolError, msg:
            self.errorMessage('Could not connect to server %s:%s as %s.' \
                              % (self.server_name, self.port, self.uid))
            self.errorMessage(msg)
            return
        message_text = string.join(args)
        if not message_text:
            self.showHelp('Specify the message contents as arguments to send a message.')
            return
        for user in self.to_address:
            try:
                self.server.mesg(user, message_text)
            except wcc_protocol.WackyProtocolError, msg:
                self.errorMessage('Could not send message to %s' % user)
                self.errorMessage(msg)
                pass
        return


if __name__ == '__main__':
    try:
        QuickWacky().run()
    except CommandLineApp.HelpRequested:
        pass
    
