# -*- perl -*-
# IO.pm $Id: Io.pm,v 1.4.2.1 2000/01/30 13:56:23 jens Exp $
# (C) Copyright Jens G Jensen <jens@argaeus.ma.rhbnc.ac.uk>
# This file is part of epsmerge and is distributed under GNU GPL

package Io;

use strict;

sub new {
    return bless {}, shift;
}

package Io_Console;

use strict;
use vars qw(@ISA);

@ISA = qw(Io);

# Basic rule for console IO:
#   Input is taken from STDIN unless STDIN is not interactive in which case it is currently an error
#   Output is written to STDOUT unless option '-o' points to stdout in which case output is written to STDERR
# No error is reported when _creating_ the Io class.
sub new {
    my $self = Io::new(@_);
    $self->{'in'} = \*STDIN;				# no error yet even if STDIN is not interactive
    $self->{'out'} = Options->new()->getopts('o') eq 'stdout' ? \*STDERR : \*STDOUT;
    return $self;
}

# Input:
# 1. A message string, possibly with newlines in it
# 2. A list of possible answers
sub ask {
    my ($self, $text) = (shift, shift);
    die "Not running interactively" unless -t $self->{'in'};
    print { $self->{'out'} } "$text\n[";
    while( @_ ) {
	my $opt = shift;
	print $opt, @_ ? "/" : "";
    }
    print { $self->{'out'} } "]: ";
    my $stdin_ref_hack = $self->{'in'};
    my $r = <$stdin_ref_hack>;				# <> doesn't like complicated things, not even {}s
    chomp $r;
    # FIXME: check that answers belong to the list of options
    return $r;
}

# Call with a message and, optionally, a priority which should be one of the following:
# 'D': debug
# 'W': warning
# 'E': error
# 'F': fatal
# Otherwise, 'info' is assumed.
sub message {
    my $self = shift;
    # for now, we print everything except debug messages
    print { $self->{'out'} } $_[0],"\n" unless defined $_[1] && $_[1] eq 'D';
}

sub status {
    # currently only for debugging purposes
    print STDERR "Status: $_[1]\n";
}


1;
