#!/bin/bash
# compile pymat on a Unix computer
# by Hans Petter Langtangen (hpl@simula.no)


# Where is the Matlab directory located on this computer system?
MATLAB_DIR=/usr/local/matlab
MATLAB_HARDWARE_TYPE=glnx86  # Linux, change for Sun, IBM, etc.

# The rest is often correct if MATLAB_DIR and MATLAB_HARDWARE_TYPE are set...

MATLAB_INCLUDE=$MATLAB_DIR/extern/include
MATLAB_LIB=$MATLAB_DIR/extern/lib/$MATLAB_HARDWARE_TYPE
PYTHON_VERSION=`python -c 'import sys; print sys.version[:3]'`
PYTHON_INSTALL=`python -c 'import sys; print sys.prefix'`
PYTHON_INCLUDE=$PYTHON_INSTALL/include/python$PYTHON_VERSION
NUMERIC_INCLUDE=$PYTHON_INCLUDE/Numeric
PYMAT_INSTALL=$PYTHON_INSTALL/lib/python$PYTHON_VERSION/site-packages
# local install of pymat instead:
PYMAT_INSTALL=$HOME/install/lib/python

INCLUDE="-I$MATLAB_INCLUDE -I$NUMERIC_INCLUDE -I$PYTHON_INCLUDE"
COMPILE="g++ -O $COMPILER_WITH_OPTIONS $INCLUDE -c pymat.cpp -o pymat.o"
LINK="g++ -shared -L$MATLAB_LIB -o pymat.so pymat.o -leng -lmx -lmat -lut  -Wl,-rpath,$MATLAB_LIB"

# clean:
rm -f pymat.o pymat.so

echo $COMPILE
$COMPILE

echo $LINK
$LINK

rm -f pymat.o
# install:
mv -f pymat.so pymat
echo "cp -r pymat $PYMAT_INSTALL"
cp -r pymat $PYMAT_INSTALL

# TROUBLESHOOTING:
#
# Matlab libraries may be linked with libraries that have other names than
# those on your computer. Do a
# ldd $MATLAB_LIB/*.so
# and create necessary links (usually in /usr/lib) from physically available
# libraries to the libraries that the Matlab .so files are linked with.
# python -c 'import pymat' often gives informative error messages in this
# regard.






