#!/usr/bin/python
"""
Python script by Tobias Polzin, polzin@gmx.de
If you like the module, please give me feedback.
All rights reserved.

<title>
txt2html.py, Version 4.5.00 (Update: 15.5.02)
</title>
<long>
Rapid creation of HTML and TEX files from txt files with the following syntax
(indenting lines is used for scoping as in PYTHON):
<pre>
>Headlines

Normal text, _bold text_, *emphasised*
there are:
 - descriptions: this is a description text
   the identation is used as in python. 
     * itemizes
     * unsorted lists
</pre>     

Converted to html:
<h2>Headlines</h2>

<br>
Normal text, <b>bold text</b>, <em>emphasised</em>
<br>
there are:
<dl>
<dt>descriptions:<dd>
    this is a description text
   the identation is used as in python. 
<ul>
<li>
       itemizes
<li>
       unsorted lists
</ul>
</dl>

Converted to Latex:
<pre>
% Generated by txt2html, polzin@gmx.de
% tt (05.05.2000)
\documentclass[a4paper]{article}
\usepackage[latin1]{inputenc}
\begin{document}


\subsection*{Headlines}

\\
Normal text, \textbf{bold text},  \emph{emphasised}
\\
there are:
\begin{description}
\item[descriptions:]
    this is a description text
   the identation is used as in python. 
\begin{itemize}
\item
       itemizes
\item
       unsorted lists
\end{itemize}
\end{description}
\end{document}
</pre>
</long>
"""
from sys import argv,exit
from string import split
import time
import re

linescan=re.compile(r"(?P<indent> *(?P<mode>[*>-])? *)"+
                    r"(?P<line>(?P<title>[^?!:.]+[?!:.])?(?P<notitle>.*))\n?")
emphscan=re.compile(r"\s\*(\S+)\*($|(?=\s))")
bfscan=re.compile("_")

def bfinsert(m):
    global bfison
    s=BF[bfison]
    bfison=1-bfison
    return s

#argv=["","/home/tobias/texte/xquer/texte/protokoll-april"]

if len(argv)<2:
    print "txt2html, Tobias Polzin, 15.5.02"
    print "usage: txt2html [-t] {filename}"
    exit(1)
    
maxlen=60
tex=0
if argv[1] in ["-t","--tex"]:
    del argv[1]
    tex=1
filename=argv[1]  

BR=['<br>',r'\\'][tex]
EM=[('<em>','</em>'),(r'\emph{',r'}')][tex]
BF=[('<b>','</b>'),(r'\textbf{',r'}')][tex]
MODE=[
    {'*':('<ul>','<li>','</ul>'),
     '-':('<dl>','<dt>%s<dd>','</dl>'),
     '>':('','<h2>%s</h2>','')},
    {'*':(r'\begin{itemize}',r'\item',r'\end{itemize}'),
     '-':(r'\begin{description}',r'\item[%s]',r'\end{description}'),
     '>':('',r'\subsection*{%s}','')}
    ][tex]
INTRO=[
r'''<html><head>
<meta name="description" content="Generated by txt2html, polzin@gmx.de">
<title>%s</title></head><body>''',
r'''%% Generated by txt2html, polzin@gmx.de
%% %s
\documentclass[a4paper]{article}
\usepackage[latin1]{inputenc}
\begin{document}
'''][tex] % (filename +
time.strftime(" (%d.%m.%Y)",time.localtime(time.time())))
EXIT=['</body></html>',r'\end{document}'][tex]

lastlen=80
lastindent=0
indents=[]
bfison=0

f=open(filename,"r")
print INTRO
for l in f.readlines():
    m=linescan.match(l)
    llen=len(l)
    indent=len(m.group("indent"))

#    print "% "+`m.group("mode")`,indent,lastindent
    if lastindent<indent and m.group("mode"):
        tag=m.group("mode")
        indents.append((tag,indent))
        print MODE[tag][0]
        lastlen=maxlen
        lastindent=indent

    if lastindent>indent:
        while indents and indents[-1][1]>indent:
            print MODE[indents[-1][0]][2]
            del indents[-1]
        lastlen=maxlen
        lastindent=indent

    line=m.group("line")
    if m.group("mode"):
        p=MODE[m.group("mode")][1]
        if "%" in p:
            if m.group("title"):
                p=p%m.group("title")
                line=m.group("notitle")
            else:
                p=p%line
                line=""
        if p:
            print p
        llen=maxlen
    elif (len(split(line))>0 and lastlen+len(split(line)[0])<maxlen):
        print BR
    line=emphscan.sub(r" %s\1%s" % (EM[0],EM[1]),line)
    line=bfscan.sub(bfinsert,line)
    if len(line):
        print " "*indent+line
        lastindent=indent
    lastlen=llen
print EXIT


