##############################################################################
# $Id: bind.tcl,v 1.5 2000/08/14 03:00:25 cgavin Exp $
#
# bind.tcl - procedures to control widget bindings
#
# Copyright (C) 1996-1998 Stewart Allen
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

##############################################################################
#

proc vTclWindow(post).vTcl.bind {args} {
    vTcl:setup_vTcl:bind .vTcl.bind
}

proc vTcl:add_bind {} {
    global vTcl
    bind $vTcl(w,widget) $vTcl(bind,event) [string trim \
            [.vTcl.bind.fra9.fra16.tex21 get 0.0 end] "\n"]
    vTcl:get_bind $vTcl(w,widget)
}

proc vTcl:delete_bind {} {
    global vTcl
    if {$vTcl(bind,event) != ""} {
        bind $vTcl(w,widget) $vTcl(bind,event) ""
    }
    vTcl:get_bind $vTcl(w,widget)
}

proc vTcl:get_bind {target} {
    global vTcl
    if {[winfo exists .vTcl.bind] == 1} {
        set vTcl(bind,list) [bind $target]
        .vTcl.bind.fra9.ent14 delete 0 end
        .vTcl.bind.fra16.tex21 delete 0.0 end
        .vTcl.bind.fra12.lis10 delete 0 end
        foreach i $vTcl(bind,list) {
            .vTcl.bind.fra12.lis10 insert end $i
        }
        set vTcl(bind,tags) $vTcl(bindtags,$target)
        set vTcl(bind,event) ""
    }
}

proc vTcl:select_bind {} {
    global vTcl
    .vTcl.bind.fra9.fra16.tex21 delete 0.0 end
    set listnum [.vTcl.bind.fra12.lis10 curselection]
    if {$listnum != ""} {
        set vTcl(bind,event) [.vTcl.bind.fra12.lis10 get $listnum]
        .vTcl.bind.fra9.fra16.tex21 insert end [bind $vTcl(w,widget) \
                $vTcl(bind,event)]
       # Nelson @RJ Ent 05/13/2000
       # syntax colouring
       vTcl:syntax_color .vTcl.bind.fra9.fra16.tex21
       # end Nelson @RJ Ent
    }
}

proc vTcl:update_bind {} {
    global vTcl
    set listnum [.vTcl.bind.fra12.lis10 curselection]
    if {$listnum != ""} {
        bind $vTcl(w,widget) $vTcl(bind,event) [string trim \
                [.vTcl.bind.fra9.fra16.tex21 get 0.0 end] "\n"]
    }
    vTcl:get_bind $vTcl(w,widget)
}

proc vTclWindow.vTcl.bind {args} {
    # Rozen.  I completely rewrote this procedure.
    # The origional relied heavily on the place command and fixed
    # coordinates, which was bad because many sizes are dependent 
    # on the size of the font selected.  Because of my tired old 
    # eyes, I need big fonts and now this will accomodate them.
    # I wonder if it was built using the tool.
    set base .vTcl.bind
    if {[winfo exists .vTcl.bind]} {
        wm deiconify .vTcl.bind; return
    }
    toplevel .vTcl.bind -class vTcl
    wm transient .vTcl.bind .vTcl
    wm focusmodel .vTcl.bind passive
    wm geometry .vTcl.bind +79+162
    wm maxsize .vTcl.bind 1137 870
    wm minsize .vTcl.bind 415 350
    wm overrideredirect .vTcl.bind 0
    wm resizable .vTcl.bind 1 0      ;# Rozen was 0 0
    wm deiconify .vTcl.bind
    wm title .vTcl.bind "Widget Bindings"
    frame .vTcl.bind.fra9 \
        -borderwidth 1 -relief sunken 
    pack .vTcl.bind.fra9 \
            -padx 2 -pady 2 -fill x -expand 1 -anchor nw
    # Top row.
    label .vTcl.bind.fra9.lab23 \
        -relief groove -text {Type -->} -width 9
    pack  .vTcl.bind.fra9.lab23 \
           -anchor nw -padx 3 -pady 3 -side left  
    entry .vTcl.bind.fra9.ent14 \
        -highlightthickness 0 -textvariable vTcl(bind,scratch) -width 15
    bind .vTcl.bind.fra9.ent14 <KeyPress> {
        append vTcl(bind,event) "<Key-%K>"
    }
    pack .vTcl.bind.fra9.ent14 \
           -anchor nw -padx 3 -pady 3 -side left
    menubutton .vTcl.bind.fra9.men1 \
        -borderwidth 1 \
        -menu .vTcl.bind.fra9.men1.m -relief raised \
        -text Mouse
    pack .vTcl.bind.fra9.men1 \
           -anchor nw -padx 3 -pady 3 -side left -expand 1 -fill x
    menubutton .vTcl.bind.fra9.men2 \
        -borderwidth 1 \
        -menu .vTcl.bind.fra9.men2.m -relief raised -text Key
    pack .vTcl.bind.fra9.men2 \
           -anchor nw -padx 3 -pady 3 -side left -expand 1 -fill x
    menubutton .vTcl.bind.fra9.men3 \
        -borderwidth 1 \
        -relief raised -text Mod \
        -menu .vTcl.bind.fra9.men3.m
    pack .vTcl.bind.fra9.men3 \
           -anchor nw -padx 3 -pady 3 -side left -expand 1 -fill x
    #Since I don't have anything to do in the window 
    # menu, I will just delete it.
###    menubutton .vTcl.bind.fra9.men4 \
###        -borderwidth 1 \
###         -padx 5 \
###        -pady 4 -relief raised -text Window
###    pack .vTcl.bind.fra9 \
###           -anchor nw -padx 3 -pady 3 -side left
#    place .vTcl.bind.fra9.men4 \
#        -x 340 -relx 0 -y 5 -rely 0 -width 60 -relwidth {} -height 20 \
#        -relheight {} -anchor nw -bordermode ignore

    frame .vTcl.bind.fra10 \
        -borderwidth 1 -relief sunken
    pack .vTcl.bind.fra10 \
            -fill x -expand 1 -side top
    label .vTcl.bind.fra10.lab24 \
        -relief groove -text Command
    pack .vTcl.bind.fra10.lab24  -fill x -anchor w -padx 3 -pady 3
    # Stuff for upper listbox
    frame .vTcl.bind.fra12 \
        -borderwidth 1 -relief sunken
    pack .vTcl.bind.fra12  -fill x -anchor w -padx 3 -pady 3

    listbox .vTcl.bind.fra12.lis10 \
        -highlightthickness 0 \
        -yscrollcommand {.vTcl.bind.fra12.scr11 set} -height 5
    pack .vTcl.bind.fra12.lis10 \
        -anchor center -expand 1 -fill both -ipadx 0 \
        -ipady 0 -padx 0 -pady 0 -side left
    bind .vTcl.bind.fra12.lis10 <Double-Button-1> {
       vTcl:select_bind
    }
    scrollbar .vTcl.bind.fra12.scr11 \
        -command {.vTcl.bind.fra12.lis10 yview} \
        -highlightthickness 0 
    pack .vTcl.bind.fra12.scr11 \
        -anchor center -expand 0 -fill y -ipadx 0 \
        -ipady 0 -padx 0 -pady 0 -side right
    # The Label and Tags section
    frame .vTcl.bind.fra13 \
        -borderwidth 1 -relief sunken -height 100 
    pack .vTcl.bind.fra13 \
            -fill x -expand 1 -side top
    label .vTcl.bind.fra13.lab25 \
        -relief groove -text Event -width 10
    grid  .vTcl.bind.fra13.lab25\
        -row 0 -column 0  -padx 5 
    entry .vTcl.bind.fra13.ent15 \
        -highlightthickness 0 -textvariable vTcl(bind,event)
    grid .vTcl.bind.fra13.ent15 \
        -row 0 -column 1  -padx 5 -sticky ew
    label .vTcl.bind.fra13.lab26 \
        -relief groove -text Tags -width 10
    grid .vTcl.bind.fra13.lab26 \
        -row 1 -column 0  -padx 5 
    entry .vTcl.bind.fra13.ent27 \
        -highlightthickness 0 -textvariable vTcl(bind,tags)
    grid .vTcl.bind.fra13.ent27 \
        -row 1 -column 1  -padx 5 -sticky ew
    grid columnconfigure .vTcl.bind.fra13 1 -weight 1
    # Middle Buttons
    frame .vTcl.bind.fra14 \
        -borderwidth 1 -relief sunken -height 100 
    pack .vTcl.bind.fra14 \
            -fill x -expand 1 -side top
    button .vTcl.bind.fra14.but13 \
        -command vTcl:add_bind \
        -highlightthickness 0 -padx 9 -pady 3 -text Add -width 5
    pack .vTcl.bind.fra14.but13 \
        -anchor center -expand 1 -fill both \
        -ipadx 0 -ipady 0 -padx 2 -pady 2 -side left
    button .vTcl.bind.fra14.but14 \
        -command vTcl:update_bind \
        -highlightthickness 0 -padx 9 -pady 3 -text Update -width 5
    pack .vTcl.bind.fra14.but14 \
        -anchor center -expand 1 -fill both \
        -ipadx 0 -ipady 0 -padx 2 -pady 2 -side left
    button .vTcl.bind.fra14.but15 \
        -command vTcl:delete_bind \
        -highlightthickness 0 -padx 9 -pady 3 -text Delete -width 5
    pack .vTcl.bind.fra14.but15 \
        -anchor center -expand 1 -fill both \
        -ipadx 0 -ipady 0 -padx 2 -pady 2 -side left
    button .vTcl.bind.fra14.but16 \
        -command {destroy .vTcl.bind} \
        -highlightthickness 0 -padx 9 -pady 3 -text Done -width 5
    pack .vTcl.bind.fra14.but16 \
        -anchor center -expand 1 -fill both \
        -ipadx 0 -ipady 0 -padx 2 -pady 2 -side left
    # Bottom text box
    frame .vTcl.bind.fra16 \
        -borderwidth 1 -relief sunken
    pack .vTcl.bind.fra16 \
            -fill x -expand 1 -side top \
            -anchor w -padx 3 -pady 3
    text .vTcl.bind.fra16.tex21 \
        -highlightthickness 0 -height 8 -width 30\
        -yscrollcommand {.vTcl.bind.fra16.scr20 set}
    pack .vTcl.bind.fra16.tex21 \
        -anchor center -expand 1 -fill both \
        -ipadx 0 -ipady 0 -padx 3 -pady 3 -side left
    scrollbar .vTcl.bind.fra16.scr20 \
        -command {.vTcl.bind.fra16.tex21 yview} \
        -highlightthickness 0
    pack .vTcl.bind.fra16.scr20 \
        -anchor center -expand 0 -fill y -ipadx 0 \
        -ipady 0 -padx 0 -pady 0 -side right
    # For the top row of menu buttons.
    menu .vTcl.bind.fra9.men1.m \
         -tearoff 0
    .vTcl.bind.fra9.men1.m add cascade \
        -label Press -menu .vTcl.bind.fra9.men1.m.press -state normal \
        -underline -1
    .vTcl.bind.fra9.men1.m add cascade \
        -label Release -menu .vTcl.bind.fra9.men1.m.release -state normal \
        -underline -1
    .vTcl.bind.fra9.men1.m add command \
        -command {append vTcl(bind,event) "<Motion>"} -label Motion \
        -state normal -underline -1
    menu .vTcl.bind.fra9.men1.m.press \
         -tearoff 0
    .vTcl.bind.fra9.men1.m.press add command \
        -command {append vTcl(bind,event) "<Button-1>"} -label 1 \
        -state normal -underline -1
    .vTcl.bind.fra9.men1.m.press add command \
        -command {append vTcl(bind,event) "<Button-2>"} -label 2 \
        -state normal -underline -1
    .vTcl.bind.fra9.men1.m.press add command \
        -command {append vTcl(bind,event) "<Button-3>"} -label 3 \
        -state normal -underline -1
    .vTcl.bind.fra9.men1.m.press add command \
        -command {append vTcl(bind,event) "<Button>"} -label any \
        -state normal -underline -1
    menu .vTcl.bind.fra9.men1.m.release \
         -tearoff 0
    .vTcl.bind.fra9.men1.m.release add command \
        -command {append vTcl(bind,event) "<ButtonRelease-1>"} -label 1 \
        -state normal -underline -1
    .vTcl.bind.fra9.men1.m.release add command \
        -command {append vTcl(bind,event) "<ButtonRelease-2>"} -label 2 \
        -state normal -underline -1
    .vTcl.bind.fra9.men1.m.release add command \
        -command {append vTcl(bind,event) "<ButtonRelease-3>"} -label 3 \
        -state normal -underline -1
    .vTcl.bind.fra9.men1.m.release add command \
        -command {append vTcl(bind,event) "<ButtonRelease>"} -label any \
        -state normal -underline -1
    menu .vTcl.bind.fra9.men2.m \
         -tearoff 0
    .vTcl.bind.fra9.men2.m add command \
        -label Press -state normal -underline -1 -command {
            append vTcl(bind,event) "<KeyPress>"
        }
    .vTcl.bind.fra9.men2.m add command \
        -label Release -state normal -underline -1 -command {
            append vTcl(bind,event) "<KeyRelease>"
        }
    menu .vTcl.bind.fra9.men3.m \
         -tearoff 0
    .vTcl.bind.fra9.men3.m add command \
        -command "vTcl:bind:set_modifier Double" \
        -label Double -state normal \
        -underline -1
    .vTcl.bind.fra9.men3.m add command \
        -command "vTcl:bind:set_modifier Triple" \
        -label Triple -state normal \
        -underline -1
    .vTcl.bind.fra9.men3.m add command \
        -command "vTcl:bind:set_modifier Control" \
        -label Control -state normal \
        -underline -1
    .vTcl.bind.fra9.men3.m add command \
        -command "vTcl:bind:set_modifier Shift" \
        -label Shift -state normal \
        -underline -1
    .vTcl.bind.fra9.men3.m add command \
        -command "vTcl:bind:set_modifier Meta" \
        -label Meta -state normal \
        -underline -1
    .vTcl.bind.fra9.men3.m add command \
        -command "vTcl:bind:set_modifier Alt" \
        -label Alt -state normal \
        -underline -1
    .vTcl.bind.fra9.men3.m add cascade \
        -label Press -menu .vTcl.bind.fra9.men3.m.press -state normal \
        -underline -1
    menu .vTcl.bind.fra9.men3.m.press \
         -tearoff 0
    .vTcl.bind.fra9.men3.m.press add command \
        -command "vTcl:bind:set_modifier Button1" \
        -label 1 \
        -state normal -underline -1
    .vTcl.bind.fra9.men3.m.press add command \
        -command "vTcl:bind:set_modifier Button2" \
        -label 2 \
        -state normal -underline -1
    .vTcl.bind.fra9.men3.m.press add command \
        -command "vTcl:bind:set_modifier Button3" \
        -label 3 \
        -state normal -underline -1

    # I don't understand this stuff so it may be wrong 
    # or not appropriate for Python. Rozen
    menu .vTcl.bind.fra16.tex21.insert -tearoff 0
    .vTcl.bind.fra16.tex21.insert add command \
    -label "%% Single percent" \
        -command ".vTcl.bind.fra16.tex21 insert insert %%"
    .vTcl.bind.fra16.tex21.insert add command \
    -label "%W Window name" \
        -command ".vTcl.bind.fra16.tex21 insert insert %W"
    .vTcl.bind.fra16.tex21.insert add command \
    -label "%b Mouse button number" \
        -command ".vTcl.bind.fra16.tex21 insert insert %b"
    .vTcl.bind.fra16.tex21.insert add command \
    -label "%A Key pressed/released (ASCII)" \
        -command ".vTcl.bind.fra16.tex21 insert insert %A"
    .vTcl.bind.fra16.tex21.insert add command \
    -label "%K Key pressed/released (Keysym)" \
        -command ".vTcl.bind.fra16.tex21 insert insert %K"
    .vTcl.bind.fra16.tex21.insert add command \
    -label "%x Mouse x coordinate widget-relative" \
        -command ".vTcl.bind.fra16.tex21 insert insert %x"
    .vTcl.bind.fra16.tex21.insert add command \
    -label "%y Mouse y coordinate widget-relative" \
        -command ".vTcl.bind.fra16.tex21 insert insert %y"
    .vTcl.bind.fra16.tex21.insert add command \
    -label "%X Mouse x coordinate desktop-relative" \
        -command ".vTcl.bind.fra16.tex21 insert insert %X"
    .vTcl.bind.fra16.tex21.insert add command \
    -label "%Y Mouse y coordinate desktop-relative" \
        -command ".vTcl.bind.fra16.tex21 insert insert %Y"
    bind .vTcl.bind.fra16.tex21 <ButtonPress-3> {
    tk_popup .vTcl.bind.fra16.tex21.insert %X %Y
    }
}

proc vTcl:bind:set_modifier {modifier} {

    global vTcl

    set last [string last < $vTcl(bind,event)]

    if {$last == -1} return

    set newbind [string range $vTcl(bind,event) 0 $last]
        set newbind $newbind${modifier}-[string range $vTcl(bind,event) [expr $last+1] end]

    set vTcl(bind,event) $newbind
}
