##############################################################################
# $Id: dragsize.tcl,v 1.3 2000/07/21 05:38:00 cgavin Exp $
#
# dragsize.tcl - procedures to handle widget sizing and movement
#
# Copyright (C) 1996-1998 Stewart Allen
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

##############################################################################
#

proc vTcl:bind_button_1 {target x y} {
    global vTcl
    # This is entered when we select a widget.
    if {[lindex [split %W .] 1] != "vTcl"} {
        if {$target != "." && [winfo class $target] != "Toplevel"} {
            vTcl:active_widget $target
            vTcl:grab $target $x $y
            set vTcl(cursor,last) [$target cget -cursor]
            $target configure -cursor fleur
        } else {
            vTcl:active_widget $target
        }
    }
}

proc vTcl:bind_button_2 {target x y} {
    global vTcl
    # Don't understand what he is trying to accomplish
    # It seems too close to vTcl:bind_button_1 to be very useful!
    if {$vTcl(w,widget) != "." && \
        [winfo class $vTcl(w,widget)] != "Toplevel" && \
        $vTcl(w,widget) != ""} {
            vTcl:grab $vTcl(w,widget) $x $y
            set vTcl(cursor,last) [$vTcl(w,widget) cget -cursor]
            $vTcl(w,widget) configure -cursor fleur
    }
}

proc vTcl:bind_ctrl_button_2 {target x y} {
    global vTcl
    # I don't know what button two does so I will create a
    # control_button_2 for my use.  I want it to put in a help balloon.
    set initial_string ""
    if {[info exists vTcl($target,balloon_msg)]} {
        set initial_string $vTcl($target,balloon_msg)
    }
    set action [vTcl:two_way_dialog "Please enter balloon message." \
                   $initial_string    "OK Cancel"]
    if {$action == "Cancel"} {
        return
    }
    set vTcl($target,balloon_msg) $vTcl(balloon_msg)
    set vTcl(balloon_messages_present) 1
    vTcl:set_balloon $target $vTcl(balloon_msg)
}

proc vTcl:bind_button_tix {target x y} { # Rozen
    global vTcl
    # What I want to do is bind <Control-Button-1> to the top tix widget
    # not one of the inside widgets. Rozen
    if {[lindex [split %W .] 1] != "vTcl"} {
        if {[string first "tix" $target] == -1} {
            # Already at tix widget.
            return
        }
        set pieces [split $target .]
        set max 0
        for {set i 1} {$i <= [llength $pieces]} {incr i} {
            set can [lindex $pieces $i]
            if {[string first "tix" $can] != -1} {
                set max $i
            }
        }
        for {set i 1} {$i <= $max} {incr i} {
            set can [lindex $pieces $i]
            append new_target "."
            append new_target $can
        }
        if {$new_target != "." && [winfo class $new_target] != "Toplevel"} {
            vTcl:active_widget $new_target
            vTcl:grab $new_target $x $y
            set vTcl(cursor,last) [$new_target cget -cursor]
            $new_target configure -cursor fleur
        } else {
            vTcl:active_widget $new_target
        }
    }
}

proc vTcl:bind_motion {x y} {
    global vTcl
    if {$vTcl(w,widget) != "." && $vTcl(w,class) != "Toplevel"} {
        vTcl:grab_motion $vTcl(w,widget) $x $y
    }
}

proc vTcl:bind_release {x y} {
    global vTcl
    if {$vTcl(w,widget) == ""} {return}
    $vTcl(w,widget) configure -cursor "$vTcl(cursor,last)"
    vTcl:place_handles $vTcl(w,widget)
    vTcl:grab_release $vTcl(w,widget)
    vTcl:update_widget_info $vTcl(w,widget)
}

proc vTcl:grab {widget absX absY} {
    global vTcl
    grab $widget
    set vTcl(w,didmove) 0
    set vTcl(w,grabbed) 1
    set vTcl(grab,startX) [vTcl:grid_snap x $absX]
    set vTcl(grab,startY) [vTcl:grid_snap y $absY]
    set vTcl(w,lastx) $vTcl(grab,startX)
    set vTcl(w,lasty) $vTcl(grab,startY)
}

proc vTcl:grab_motion {widget absX absY} {
    global vTcl

    set vTcl(w,didmove) 1
    # workaround for Tix
    if { $vTcl(w,grabbed) == 0 } {
        vTcl:grab $widget $absX $absY
    }
    if { $vTcl(w,manager) == "place" } {

        set last_x [vTcl:grid_snap x $absX]
        set last_y [vTcl:grid_snap y $absY]
        place $widget \
            -x [vTcl:grid_snap x \
                [expr {$absX-$vTcl(grab,startX)+$vTcl(w,x)}]] \
            -y [vTcl:grid_snap y \
                [expr {$absY-$vTcl(grab,startY)+$vTcl(w,y)}]]
        set vTcl(w,lastx) $last_x
        set vTcl(w,lasty) $last_y
    }
    vTcl:place_handles $widget
}

proc vTcl:grab_release {widget} {
    global vTcl
    #if {$vTcl(w,pane_sep_found)} {
    #    # We are trying to move a pane boundry.
    #    # The variable above was set in vTcl:select_widget.
    #    vTcl:resize_panes $widget
    #}
    grab release $widget
    set vTcl(w,grabbed) 0
    if { $vTcl(w,didmove) == 1 } {
        set vTcl(undo) [vTcl:dump_widget_quick $vTcl(w,widget)]
        vTcl:passive_push_action $vTcl(undo) $vTcl(redo)
    }
}

# The stuff below does not handle compound widgets!!!!
proc vTcl:grab_resize {absX absY handle} {
    global vTcl
    set vTcl(w,didmove) 1
    set widget $vTcl(w,widget)
#puts "vTcl:grab_resize: widget = $widget"
#puts "vTcl:grab_resize: class = $vTcl(w,class)"
    set X [vTcl:grid_snap x $absX]
    set Y [vTcl:grid_snap y $absY]
    set deltaX [expr {$X - $vTcl(grab,startX)}]
    set deltaY [expr {$Y - $vTcl(grab,startY)}]
    switch $vTcl(w,manager) {
        place {
#puts "grab_resize: in place"
            switch $handle {
                n {
                    set newX $vTcl(w,x)
                    set newY [expr {$vTcl(w,y) + $deltaY}]
                    set newW $vTcl(w,width)
                    set newH [expr {$vTcl(w,height) - $deltaY}]
                }
                e {
                    set newX $vTcl(w,x)
                    set newY $vTcl(w,y)
                    set newW [expr {$vTcl(w,width) + $deltaX}]
                    set newH $vTcl(w,height)
                }
                s {
                    set newX $vTcl(w,x)
                    set newY $vTcl(w,y)
                    set newW $vTcl(w,width)
                    set newH [expr {$vTcl(w,height) + $deltaY}]
                }
                w {
                    set newX [expr {$vTcl(w,x) + $deltaX}]
                    set newY $vTcl(w,y)
                    set newW [expr {$vTcl(w,width) - $deltaX}]
                    set newH $vTcl(w,height)
                }
                nw {
                    set newX [expr {$vTcl(w,x) + $deltaX}]
                    set newY [expr {$vTcl(w,y) + $deltaY}]
                    set newW [expr {$vTcl(w,width) - $deltaX}]
                    set newH [expr {$vTcl(w,height) - $deltaY}]
                }
                ne {
                    set newX $vTcl(w,x)
                    set newY [expr {$vTcl(w,y) + $deltaY}]
                    set newW [expr {$vTcl(w,width) + $deltaX}]
                    set newH [expr {$vTcl(w,height) - $deltaY}]
                }
                se {
                    set newX $vTcl(w,x)
                    set newY $vTcl(w,y)
                    set newW [expr {$vTcl(w,width) + $deltaX}]
                    set newH [expr {$vTcl(w,height) + $deltaY}]
                }
                sw {
                    set newX [expr {$vTcl(w,x) + $deltaX}]
                    set newY $vTcl(w,y)
                    set newW [expr {$vTcl(w,width) - $deltaX}]
                    set newH [expr {$vTcl(w,height) + $deltaY}]
                }
            }
            #place $widget -x $newX -y $newY  -width $newW -height $newH
            place $widget -x $newX -y $newY ;# -width $newW -height $newH
            # The above comment seems to clear up the drag problem that I
            # ran into,
            # i. e., when you drag an entry box then you cannot adjust the width
            # using the attribute editor. Rozen
        }

        grid -
        pack {
#puts "grab_resize: in grid or pack"
            switch $vTcl(w,class) {
                Label -
                Entry -
                Message -
                Scrollbar -
                Scale {
                    set vTcl(w,opt,-height) ""  ;# Rozen he commented out
                }
            }

            switch $handle {
                n {
                    set newW $vTcl(w,opt,-width)
                    set newH [expr {$vTcl(w,opt,-height) - $deltaY}]
                }
                e {
                    set newW [expr {$vTcl(w,opt,-width) + $deltaX}]
                    set newH $vTcl(w,opt,-height)
                }
                s {
                    set newW $vTcl(w,opt,-width)
                    set newH [expr {$vTcl(w,opt,-height) + $deltaY}]
                }
                w {
                    set newW [expr {$vTcl(w,opt,-width) - $deltaX}]
                    set newH $vTcl(w,opt,-height)
                }
                nw {
                    set newW [expr {$vTcl(w,opt,-width) - $deltaX}]
                    set newH [expr {$vTcl(w,opt,-height) - $deltaY}]
                }
                ne {
                    set newW [expr {$vTcl(w,opt,-width) + $deltaX}]
                    set newH [expr {$vTcl(w,opt,-height) - $deltaY}]
                }
                se {
                    set newW [expr {$vTcl(w,opt,-width) + $deltaX}]
                    set newH [expr {$vTcl(w,opt,-height) + $deltaY}]
                }
                sw {
                    set newW [expr {$vTcl(w,opt,-width) - $deltaX}]
                    set newH [expr {$vTcl(w,opt,-height) + $deltaY}]
                }
            }
        }
    }
            if { $newW < 0 } { set newW 0 }
            if { $newH < 0 } { set newH 0 }
            # @@change by Christian Gavin 3/19/2000
            # added catch in case some widgets don't have a -width
            # or a -height option (for example Iwidgets toolbar)
            catch {
            switch $vTcl(w,class) {
                Label -
                Entry { # Rozen
                    set this_font [$vTcl(w,widget) cget -font]
                    set cx [font measure $this_font 0]
                    $widget configure -width [expr $newW / $cx]
                }
                Message -
                TixMeter -
                Scrollbar {
                    $widget configure -width $newW
                }
                Scale {
                    $widget configure -length $newW  ;# Rozen
                }
                Text {
                    set this_font [$vTcl(w,widget) cget -font]
                    set cx [font measure $this_font 0]
                    set cy [font metrics $this_font -linespace]
                    $widget configure -width [expr $newW / $cx]  \
                                      -height [expr $newH / $cy]
                }
                TixNoteBook {
                    foreach i  [$widget pages] {
                        $widget.nbframe.$i configure -width $newW -height $newH
                    }
                    $widget configure -width $newW -height $newH
                }
                TixLabelFrame { # Rozen So I can drag a Tix widget.
                    $widget.border.frame configure -width $newW -height $newH
                }
                TixPanedWindow { # Rozen
                    $widget configure -width $newW -height $newH
                    vTcl:adjust_panes $widget
                }
                default {
                    $widget configure -width $newW -height $newH
                }
            }
        }
            # @@end_change
    vTcl:place_handles $widget
}


