#
# python_ui.tcl - procedures for generating Python User Interfaces
#
# This program was written by Don Rozenberg and is based on java_ui.tcl
# written by Constantin Teodorescu.
#
# The original copyright notice is attached and this program is released
# under the terms of that copywrite.
#
# Copyright (C) 1996-1997 Constantin Teodorescu
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

##############################################################################

#  To  add a special attribute:
#      1. Add the attribute to global.tcl or lib_tix to put it in
#          vTcl(opt,list)
#      2. Add a new case to vTcl:update_widget_info
#      3. Add code to case of  vTcl:python_source_widget to handle
#             the special attributes.


proc vTcl:generate_python_UI {} {
    global vTcl env
    set window $vTcl(w,widget)
    if {($window=="") || ($window==".")} {
        #bell
        if {[llength $vTcl(tops)] != 1} {
            tk_messageBox -title Error -message "Please select first a window"
            return
        } else {
            set window $vTcl(tops)
        }
    }
    set cite 0
    while {1} {
        incr cite
        if {$cite>10} {break}
        if {[llength [split $window .]]==2} { break}
        set window [winfo parent $window]
    }
    set vTcl(py_window) $window
    set vTcl(py_classname) [wm title $window]
    if {$vTcl(pr,emacs) == 0} {
        vTclWindow.vTcl.py_console
        update
        #.vTcl.py_console.f3.t delete 1.0 end
        #.vTcl.py_console.f5.t delete 1.0 end
        #.vTcl.py_console.pane.text1.text1 subwidget text delete 1.0 end
        #.vTcl.py_console.pane.text2.text2 subwidget text delete 1.0 end
        $vTcl(source) subwidget text delete 1.0 end
        $vTcl(output) subwidget text delete 1.0 end
    }
    vTcl:save             ;# This saves the tcl version.
    vTcl:generate_python
}

proc vTcl:generate_python {} {
    global vTcl
    set vTcl(menubutton_list) {}
    set window $vTcl(py_window)
    # Get window geometry
    foreach {W H X Y} [split [winfo geometry $window] "x+"] {}
    incr H 28
    incr W 7
    set resizable [wm resizable $window]
    if {([lindex $resizable 0]=="0") || ([lindex $resizable 1]=="0")} {
        set resizable false
    } else {
        set resizable true
    }
    set classname $vTcl(py_classname)
    if {$vTcl(pr,emacs) == 0} {
        set constructor_arguments ""
        set constructor_super ""
        set main_args ""
    }
    set main {}
    if {$vTcl(pr,emacs) == 0} {
        set onWindowClose "this.setVisible(false); this.dispose();"
    }

    set lwin $vTcl(py_window)
    set geom [winfo geometry $window]
    set main [vTcl:python_gui_startup $geom $classname]
    append main "\nif __name__ == '__main__':\n"
    append main "$vTcl(tab)vp_start_gui()\n"
    set onWindowClose "sys.exit()"

    set vTcl(py_vars) {}
    set vTcl(py_initfunc) {}
    set vTcl(py_funcdef) {}
    set vTcl(py_action) {}
    set vTcl(py_has_menus) 0
    foreach bg [array names vTcl py_btngroup,*] {
        unset vTcl($bg)
    }
    vTcl:py_source_frame $window   ""  ""
    vTcl:py_process_menubutton_list
    set my_procs ""
    set class_procs "\n"

    # If an init proc has not been defined then build a null version.
    if {[string trim [info body "init"]] == ""} {
        proc init {} {def init(): pass}
        lappend vTcl(procs) "init"
    }
    # Separate the proc's into those that go inside the class definition
    # from those that go outside.
    foreach i  [lsort $vTcl(procs)] {
        if {$i == "main"} {
            continue
        }
        if {$i == "vp_start_gui"} {
            continue
        }
        if {$i == "append_python_attributes"} {
            continue
        }
        if {![regexp -nocase {vtcl} $i]} {
            # NEEDS WORK - following test just isn't working.
            # First of all, $i is just the name. not the first line!
            set new_proc [info body $i]
            if {[string first "self." $new_proc] == -1} {
                # Global proc
                set new_proc [vTcl:python_subst_tabs $new_proc]
                set my_procs "$my_procs$new_proc\n"
            } else {
                # class procs
                #vTcl:python_print_proc $new_proc
                #set new_proc [string range $new_proc 1 end]
                set new_proc [vTcl:python_subst_tabs $new_proc]
                set new_proc [vTcl:python_delete_leading_tabs $new_proc]
                set new_proc [vTcl:python_indent_lines $new_proc]
                set class_procs "$class_procs$new_proc\n"
            }
        }
    }
    # Now I build the balloon stuff.
    set balloon_code [vTcl:create_balloon_code $vTcl(tops)]

    set source \
"#! /usr/bin/env python
from Tkinter import *
import Tix, sys
root = Tix.Tk()

$my_procs

class $classname:
    def __init__(self, master=None):
$vTcl(py_initfunc)"
set source "$source
$balloon_code
$class_procs
$main
"
    if {$vTcl(pr,emacs) == 0} {
        #.vTcl.py_console.f3.t delete 1.0 end
        #.vTcl.py_console.f3.t insert end $source
        $vTcl(source) subwidget text delete 1.0 end
        $vTcl(source) subwidget text insert end $source
    } else {
        #The following writes to a file.
        set fn "[file rootname $vTcl(project,file)].py"
        set f [open $fn w]
        puts $f $source
        close $f
        exec emacsclient $fn
        # End of write.
    }
    set vTcl(py_source) $source
    return
}

proc vTcl:python_save_py_file {} {
    global vTcl
    # First we save the tcl file and then the python file.
    vTcl:save
    set filename "[file rootname $vTcl(project,file)].py"
    update
    update idletasks
    set fid [open $filename w]
    puts $fid $vTcl(py_source)
    flush $fid
    close $fid
    return 0
}

proc vTcl:py_source_frame {window prefix parentframe} {
    global vTcl
    if {$prefix!=""} {
        set prefix $prefix\_
    }
#    set menu_y_shift 0
#    foreach widget [winfo children $window] {
#        if {[winfo class $widget]=="Menubutton"} {
#            set menu_y_shift 24
#            break
#        }
#    }
    set vTcl(balloon_widgets) ""
    foreach widget [winfo children $window] { # Rozen
        vTcl:python_source_widget $widget $prefix $parentframe
    }
}



proc vTcl:py_get_command_parameters {cmdpar} {
    # I have no idea what this routine does. Rozen
    # But I clearly don't need to check for comments beginning
    # with //
    global vTcl
    upvar cmdname lname
    upvar cmdbody lbody
    set lname {}
    set lbody {}
    catch {
        set lname $cmdpar
        if {$lname!=""} {
            # Check if lname already contains Java code
            # the first line should end with a ;
            # or the first non empty line begins with a comment //
            foreach line [split $lname "\n"] {
                set line [string trim $line]
                if {([string range $line 0 1]=="//")
                      || ([string range $line end end]==";")} {
puts "py_get_command_parameters: Bingo!"
[expr 1 / 0]
                    set lbody $lname
                    set lname {}
                    return
                }
                if {[info procs $lname]!=""} {
                    set lbody [info body $lname]
                }
            }
        }
    }
}

proc vTcl:python_parse_menu {menubutton widname {suffix m}} {
    global vTcl
    foreach widget [winfo children $menubutton] {
        if {[winfo class $widget]=="Menu"} {
            set nritems [$widget index last]
            for {set i 0} {$i<=$nritems} {incr i} {
                switch [$widget type $i] {
                    command {
                        vTcl:py_get_command_parameters \
                                [$widget entrycget $i -command]
                        if {$cmdname==""} {
                            set cmdname "$widname\_$i\_click"
                        }
                        append vTcl(py_initfunc) \
                         "$vTcl(tab2)self.$widname\_m$suffix.add_command(label="
                        append vTcl(py_initfunc) \
                                "\"[$widget entrycget $i -label]\","
                        append vTcl(py_initfunc) \
                                "command=$cmdname,underline=0)\n"
                    }
                    checkbutton { # Like to put in more options
                        vTcl:py_get_command_parameters \
                                [$widget entrycget $i -command]
                        if {$cmdname == ""} {
                            append vTcl(py_initfunc) \
        "$vTcl(tab2)self.$widname\_m$suffix.add_checkbutton(label=\"[$widget entrycget $i -label]\")\n"
                        } else {
                            append vTcl(py_initfunc) \
        "$vTcl(tab2)self.$widname\_m$suffix.add_checkbutton(label=\"[$widget entrycget $i -label]\",command=$cmdname)\n"
                        }
                    }
                    radiobutton { # Like to put in more options
                        vTcl:py_get_command_parameters \
                                [$widget entrycget $i -command]
                        if {$cmdname == ""} {
                            append vTcl(py_initfunc) \
        "$vTcl(tab2)self.$widname\_m$suffix.add_radiobutton(label=\"[$widget entrycget $i -label]\")\n"
                        } else {
                                append vTcl(py_initfunc) \
        "$vTcl(tab2)self.$widname\_m$suffix.add_radiobutton(label=\"[$widget entrycget $i -label]\",command=$cmdname)\n"
                        }
                    }
                    separator {
                        append $vTcl(py_initfunc)  \
                         "$vTcl(tab2)self.$widname\_m$suffix.add_separator({})\n"
                    }
                    cascade {  # NEEDS WORK due to change in suffix.
                        #set incsuffix [expr $suffix + 1]  ;# NEEDSWORK
                        set incsuffix ""
                        append incsuffix "m$suffix" "_cas$suffix"
                        append vTcl(py_initfunc) \
                                "$vTcl(tab2)self.$widname\_$incsuffix = "
                        append vTcl(py_initfunc) \
                                "Menu(self.$widname\_m$suffix)\n"
                        append vTcl(py_initfunc) \
                                "$vTcl(tab2)self.$widname\_m$suffix.add_cascade(label=\"[$widget entrycget $i -label]\",menu=self.$widname\_$incsuffix)\n"
                        set arg_suffix ""
#append vTcl(py_initfunc) "# cascade arg_suffix = $arg_suffix\n"
                        append arg_suffix $suffix "_cas$suffix"
                        vTcl:python_parse_menu $widget $widname \
                                $arg_suffix

#                                [expr $suffix + 1]
                    }
                }
            }
        }
    }
}

proc vTcl:python_parse_combo_menu {menubutton widname} {
    global vTcl
    global suffix
    foreach widget [winfo children $menubutton] {
        set suffix [expr $suffix + 1]
        if {[winfo class $widget]=="Menu"} {
            set nritems [$widget index last]
            for {set i 0} {$i<=$nritems} {incr i} {
                set suffix [expr $suffix + 1]
                if {[$widget type $i]=="command"} {
                    vTcl:py_get_command_parameters \
                            [$widget entrycget $i -command]
                    append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname\.add_command(\"z_$suffix\",label="
                    append vTcl(py_initfunc) \
                            "\"[$widget entrycget $i -label]\","
                    append vTcl(py_initfunc) \
                            "underline=0)\n"
                }
                if {[$widget type $i]=="separator"} {

                    append vTcl(py_initfunc) \
                            "$vTcl(tab)$vTcl(tab)self."
                    append vTcl(py_initfunc) \
                            "$widname\.add_separator(\"z_$suffix\")\n"
                }
            }
        }
    }
}

proc vTcl:python_quick_run {} {
global vTcl
    update
    if {[vTcl:python_save_py_file]} {
        return
    }
    update
    vTcl:python_run
}

proc vTcl:python_run {} {
    global vTcl
    set root_name [file rootname $vTcl(project,file)]
    set pc .vTcl.py_console
    $pc configure -cursor watch
    set pc_output $vTcl(output)
    $pc_output subwidget text delete 1.0 end
    $pc_output subwidget text insert end "Running $root_name.py ...\n"
    #$pc_output tag configure err -background yellow
    update
    file delete $root_name.output
    catch {set res \
            "[exec python $root_name.py >>& $root_name.output]"}
    if {[file exists $root_name.output] && [file size $root_name.output] > 0} {
        set fid [open $root_name.output r]
        while {![eof $fid]} {
            gets $fid line
            $pc_output subwidget text insert end "$line\n"
        }
        close $fid
    } else {
        $pc_output subwidget text insert end "No Output"
    }
    $pc configure -cursor {}
    update
}

proc vTclWindow.vTcl.py_console {} {
    global vTcl
    set base .vTcl.py_console
    if {[winfo exists $base]} {
        wm deiconify $base; raise .vTcl.py_console
        return
    }
    toplevel $base -class Toplevel
    wm focusmodel $base passive
    wm geometry $base 775x675+100+0
    wm maxsize $base 1000 738
    wm minsize $base 500 400
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base "Python console"

    tixPanedWindow $base.pane -paneborderwidth 0
    set p1 [$base.pane add text1 -min 30 -size 500]
    set p2 [$base.pane add text2 -min 30 -size 50]
    tixScrolledText $p1.text1 -scrollbar auto
    tixScrolledText $p2.text2 -scrollbar auto
    set vTcl(source) $p1.text1
    set vTcl(output) $p2.text2
    # f4 is the frame containing the buttons.
    frame $base.f4 -height 75 -relief groove -width 125
    button $base.f4.b1 \
        -padx 9 -pady 3 -text {Build source} \
        -command vTcl:generate_python
    button $base.f4.b5 \
        -padx 9 -pady 3 -text Save \
        -command "vTcl:python_save_py_file"
    button $base.f4.b3 \
        -padx 9 -pady 3 \
        -text Run -command "vTcl:python_quick_run"
    button $base.f4.b4 \
        -padx 9 -pady 3 -text Close -command "Window hide $base"
    pack $base.pane \
        -in $base -anchor center -expand yes -fill both -padx 10 -pady 10
    pack $p1.text1 -expand yes -fill both -padx 4 -pady 6
    pack $p2.text2 -expand yes -fill both -padx 4 -pady 6
    pack $base.f4 \
        -in $base -anchor center -expand 0 -fill x -side bottom
    pack $base.f4.b1 \
        -in $base.f4 -anchor center -expand 0 -fill none -side left
    pack $base.f4.b5 \
        -in $base.f4 -anchor center -expand 0 -fill none -side left
    pack $base.f4.b3 \
        -in $base.f4 -anchor center -expand 0 -fill none -side left
    pack $base.f4.b4 \
        -in $base.f4 -anchor center -expand 0 -fill none -side right
}

proc vTcl:python_configure_widget {widget widname cmdname widclass {name ""}} {
    global vTcl
    # Obtains a list of all attributes of the widget, checks to see if
    # the current value differs from the default value.  If it does, then
    # puts adds the configuration statement for that value.
    set config [$widget configure]
    # Above returs a list of lists. The values are:
    #              argvName,  dbName,  dbClass,  defValue, and current value
    if {$name == ""} {
        set name "self.$widname"
    }
#puts "python_configure_widget: widget = $widget"
#puts "                         widname = $widname"
#puts "                         widclass = $widclass"
#puts "                         cmdname = $cmdname"
    foreach c  $config {
#puts "python_configure_widget: c = $c"
        if {[llength $c] == 5} {
            if {([lindex $c 1] == "command") && ($cmdname != "")} {
                append vTcl(py_initfunc) \
                        "$vTcl(tab2)$name.configure(command=$cmdname)\n"
            }
            if {([lindex $c 1] == "value") && ($widclass == "TixOptionMenu")} {
                continue

            }
            if {[lindex $c 3] != [lindex $c 4]} {
                set str [lindex $c 0]
                set len [string length $str]
                set sub [string range $str 1 [expr $len -1]]
                # the following switch used to have [lindex $c 1] which
                # started with '-'.  Now the rest of this stuff NEEDS WORK.
                switch -exact -- $sub {
                    padX -
                    padY -
                    dropdown -
                    editable -
                    fancy -
                    xscrollcommand -
                    yscrollcommand -
                    command {

                    }
                    textVariable {
                        switch $widclass {
                            TixLabelEntry {
                                set n $name
                            }
                            default {
                                set n "self.$widname"
                            }
                       }
                        append vTcl(py_initfunc) \
                                "$vTcl(tab2)[lindex $c 4] = StringVar()\n"
                        append vTcl(py_initfunc) \
                                "$vTcl(tab2)$n.configure($sub="
                        append vTcl(py_initfunc) \
                                "[lindex $c 4])\n"
                    }
                    variable {
                        switch $widclass {
                            "Scale" {
                                append vTcl(py_initfunc) \
                                      "$vTcl(tab2)[lindex $c 4] = DoubleVar()\n"
                                append vTcl(py_initfunc) \
                                      "$vTcl(tab2)self.$widname.configure($sub="
                                append vTcl(py_initfunc) \
                                        "[lindex $c 4])\n"
                            }
                        }
                    }
                    openCmd -
                    closeCmd -
                    browseCmd {
                        append vTcl(py_initfunc) \
                     "$vTcl(tab2)$name.configure($sub=[lindex $c 4])\n"
                    }
                    default {
                        append vTcl(py_initfunc) \
                     "$vTcl(tab2)$name.configure($sub=\"[lindex $c 4]\")\n"
                    }
                }
            }
        }
    }
}

proc vTcl:python_add_special_opt_to_optlist {option target} {
    global vTcl
    lappend vTcl(w,optlist) $option
    set vTcl(w,opt,$option) ""
    if {[info exists vTcl(w,opt,$option,$target)] != 0} {
        set vTcl(w,opt,$option) $vTcl(w,opt,$option,$target)
        if {[info exists vTcl(special_attributes)] == 0} {
            lappend vTcl(special_attributes) vTcl(w,opt,$option,$target)
        } else {
            if {[lsearch $vTcl(special_attributes) \
                    vTcl(w,opt,$option,$target)] == -1} {
                lappend vTcl(special_attributes) vTcl(w,opt,$option,$target)
            }
        }
    }
}

# Utilitiy  Similar to vTcl:lib_tix:dump_subwidgets.
# Generate source code for the subwidgets of a megawidget.
proc vTcl:python_source_subwidgets {subwidget parentframe} {
    global vTcl
    #set widget_tree [vTcl:widget_tree $subwidget]
    # The above nets me all of the widgets in the subtree.
    # Things appeared at two levels in NoteBook pages, for instance.
    # However, if a subwidget is a container widget, I
    # expect it to handle its own subwidgets.  Therefore,
    # I do not want this proceedure to look inside a
    # container subwidget.  I think I will try the following:
    set widget_tree [vTcl:get_children $subwidget]
    foreach i $widget_tree {
        set basename [vTcl:base_name $i]
        # don't try to source subwidget itself
        if {"$i" != "$subwidget"} {
            set class [vTcl:get_class $i]
            set pc [vTcl:get_class $parentframe]
            switch $pc {
                "TixLabelFrame" {
                    #set pframe [vTcl:python_get_inner_frame $parentframe]
                    set pframe [ vTcl:widget_2_widname $subwidget]
                }
                default {
                    set pframe $parentframe
                }
            }
            #set pframe $parentframe

            vTcl:python_source_widget $i ""  $pframe
        }
    }
    return
}

proc vTcl:python_source_widget {target prefix parentframe} {
    global vTcl widget
    global suffix
    # Get target name and class
    set suffix 0
    set widclass [winfo class $target]
    set widname [vTcl:widget_2_widname $target]
    if {[string first "vTH_" $target]>-1} {
        return
    }
    if {[winfo class $target]=="Menubutton"} {
        set menu_y_shift 24
    } else {
        set menu_y_shift 0
    }
    foreach {w h x y} [split [winfo geometry $target] "x+"] {}
    set rectangle "$x, [expr {$y-$menu_y_shift}], $w, $h"

    set cmdbody {}
    set cmdname {}
    catch {
        vTcl:py_get_command_parameters [$target cget -command]
    }
    if {$parentframe == ""} {     # Rozen
        set pframe "master"
    } elseif {[string first ".page" $parentframe] > -1} {
        # Another special case for the note book.
        set i  [string first ".page" $parentframe]
        set pframe "self[string range $parentframe $i end]"
    } else {
        set pframe "self.$parentframe"
    }
    foreach {w h x y} [split [winfo geometry $target] "x+"] {}

    set vTcl(py_initfunc) "$vTcl(py_initfunc)\n"
    switch $widclass {
        Label {
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname = Label ($pframe)\n"
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
            switch [$target cget -borderwidth] {
                0 {set bdt none}
                1 {set bdt SoftBevelBorder}
                default {set bdt BevelBorder}
            }
            vTcl:python_configure_widget $target $widname "" $widclass
        }
        Entry {
            vTcl:py_get_command_parameters [$target cget -textvar]
            if {$cmdname == ""} {
                set cmdname "$widname\_hit_enter"
            }
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname = Entry ($pframe)\n"
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
            vTcl:python_configure_widget $target $widname "" $widclass
        }
        Button {
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname = Button ($pframe)\n"
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
            if {$cmdname==""} {
                set cmdname "$widname\_click()"
            }
            vTcl:python_configure_widget $target $widname $cmdname $widclass
        }
        Scale {
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname = Scale ($pframe)\n"
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
            vTcl:python_configure_widget $target $widname $cmdname $widclass
        }
        Scrollbar { # Since I am using the Tix widgets I am not too interested
                    # in scrollbars, so I haven't checked them out.
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname = Scrollbar ($pframe)\n"
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
            vTcl:python_configure_widget $target $widname $cmdname $widclass
        }
        Frame {
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname = $widclass ($pframe)\n"
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
            switch [$target cget -borderwidth] {
                0 {set bdt none}
                1 {set bdt SoftBevelBorder}
                default {set bdt BevelBorder}
            }
            if {$bdt == "none"} {
                set vTcl(py_initfunc) "$vTcl(py_initfunc)$vTcl(tab)"

                    "$widname.setBorder(new EmptyBorder(0,0,0,0))\n"
            } else {
                switch [$target cget -relief] {
                    raised {
                      append vTcl(py_initfunc) \
                       "$vTcl(tab2)self.$widname.configure(relief=RAISED)\n" }
                    sunken {
                      append vTcl(py_initfunc) \
                       "$vTcl(tab2)self.$widname.configure(relief=SUNKEN)\n" }
                    groove {
                      append vTcl(py_initfunc) \
                       "$vTcl(tab2)self.$widname.configure(relief=GROOVE)\n" }
                    ridge {
                      append vTcl(py_initfunc)  \
                        "$vTcl(tab2)self.$widname.configure(relief=RIDGE)\n" }
                }

            }
            vTcl:python_configure_widget $target $widname $cmdname $widclass
            vTcl:py_source_frame $target \
                    $prefix[lindex [split $target .] end] $widname
        }
        Text {
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname = Text ($pframe)\n"
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
            vTcl:python_configure_widget $target $widname $cmdname $widclass
            vTcl:python_load_n_bind_cmds $target $widname
        }
        Listbox {
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname = Listbox ($pframe)\n"
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
            if {$cmdname==""} {
                set cmdname "$widname\_state_changed"
            }
            vTcl:python_configure_widget $target $widname $cmdname $widclass
            vTcl:python_load_n_bind_cmds $target $widname
        }
        Radiobutton {
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname = Radiobutton ($pframe)\n"
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
            if {$cmdname==""} {
                set cmdname "$widname\_click"
            }
            vTcl:python_configure_widget $target $widname $cmdname $widclass
            set rbvar [$target cget -variable]
            if {![info exists vTcl(py_btngroup,$rbvar)]} {
                set vTcl(py_btngroup,$rbvar) 1
            }
        }
        Checkbutton {
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname = Checkbutton ($pframe)\n"
            append vTcl(py_initfunc) \
                "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
            if {$cmdname==""} {
                set cmdname "$widname\_state_changed"
            }
            vTcl:python_configure_widget $target $widname $cmdname $widclass
        }
        Menubutton {
            # Just add the widget to the menubutton list.
            lappend vTcl(menubutton_list) $target
        }
        Canvas {
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname = Canvas ($pframe)\n"
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
            vTcl:python_configure_widget $target $widname $cmdname $widclass
        }
        TixLabelFrame {
            append vTcl(py_initfunc) \
                  "$vTcl(tab2)self.$widname = Tix.LabelFrame($pframe)\n"
            append vTcl(py_initfunc) \
                "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
            vTcl:python_configure_widget $target $widname ""  \
                              [winfo class $target]
            # Now I got handle the frame and the label inside.
            set sub_name [join [concat "self." $widname "_border_frame"] ""]
            append vTcl(py_initfunc) \
             "$vTcl(tab2)$sub_name = self.$widname.subwidget_list\[\"frame\"\]\n"
            set i [$target subwidget frame]
            set pf $i
            vTcl:python_configure_widget $i $widname "" $widclass $sub_name
            set sub_name [join [concat "self." $widname "_label"] ""]
            append vTcl(py_initfunc) \
            "$vTcl(tab2)$sub_name = self.$widname.subwidget_list\[\"label\"\]\n"
            set i [$target subwidget label]
            vTcl:python_configure_widget $i $widname "" $widclass $sub_name
            # Now for subwidgets inside of the TixLabelFrame.
            vTcl:python_source_subwidgets $pf $target
       }
       TixScrolledListBox {
           append vTcl(py_initfunc) \
                   "$vTcl(tab2)self.$widname = Tix.ScrolledListBox($pframe)\n"
           append vTcl(py_initfunc) \
                   "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
           vTcl:python_configure_widget $target $widname ""  \
                   [winfo class $target]
           set sub_name [join [concat $widname "_listbox"] ""]
           # Now I got handle the frame and the label inside.
           append vTcl(py_initfunc) \
             "$vTcl(tab2)self.$sub_name = "\
             "self.$widname.subwidget_list\[\"listbox\"\]\n"
           set i [$target subwidget listbox]
           vTcl:python_configure_widget $i $widname $ $widclass "self.$sub_name"
           vTcl:python_set_alias_object $target "listbox" $sub_name
           # Here we need the special attributes.
           set i [$target subwidget listbox]
           vTcl:python_load_n_bind_cmds $i $sub_name
        }
       TixOptionMenu {
           set suffix [expr $suffix + 1]
           append vTcl(py_initfunc) \
                   "$vTcl(tab2)self.$widname = Tix.OptionMenu($pframe)\n"
           append vTcl(py_initfunc) \
                   "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
           vTcl:python_configure_widget $target $widname ""  \
                   [winfo class $target]
           # Now I got handle the menu items.
           set i [$target subwidget menubutton]

           vTcl:python_parse_combo_menu $i $widname

       }
       TixLabelEntry {
           set suffix [expr $suffix + 1]
           append vTcl(py_initfunc) \
                   "$vTcl(tab2)self.$widname = Tix.LabelEntry($pframe)\n"
           append vTcl(py_initfunc) \
                   "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
           vTcl:python_configure_widget $target $widname ""  \
                   [winfo class $target]
           # Now I got handle the entry and the label inside.
           set sub_name [join [concat $widname "_frame_entry"] ""]
           append vTcl(py_initfunc) \
                   "$vTcl(tab2)self.$sub_name = "\
                   "self.$widname.subwidget_list\[\"entry\"\]\n"
           set i [$target subwidget entry]
           set pf $i   ;# What do I use this for????
           vTcl:python_configure_widget $i $widname $ $widclass "self.$sub_name"
           set sub_name [join [concat $widname "_label"] ""]
           append vTcl(py_initfunc) \
                   "$vTcl(tab2)self.$sub_name = "\
                   "self.$widname.subwidget_list\[\"label\"\]\n"
           set i [$target subwidget label]
           vTcl:python_configure_widget $i $widname $ $widclass "self.$sub_name"
       }
       TixComboBox {
           set drop [vTcl:find_attribute $target "-dropdown"]
           set edit [vTcl:find_attribute $target "-editable"]
           set fancy [vTcl:find_attribute $target "-fancy"]
           append vTcl(py_initfunc) \
                   "$vTcl(tab2)self.$widname = Tix.ComboBox($pframe,"
           append vTcl(py_initfunc) \
                   "dropdown=$drop,editable=$edit,fancy=$fancy)\n"
           #append vTcl(py_initfunc) \
           #        "$vTcl(tab2)self.$widname = Tix.ComboBox($pframe)\n"
           #append vTcl(py_initfunc) \
           #        "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
           vTcl:python_configure_widget $target $widname ""  \
                   [winfo class $target]
           # Here we need the special attributes.
           vTcl:python_load_n_bind_cmds $target $widname
       }
       TixTree {
           append vTcl(py_initfunc) \
                   "$vTcl(tab2)self.$widname = Tix.Tree($pframe,"
           # Lets try to put out options for the hlist subwidget
           set opts [vTcl:get_subwidget_options $target hlist]
           append vTcl(py_initfunc) "options='$opts')\n"
           append vTcl(py_initfunc) \
                   "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
           vTcl:python_configure_widget $target $widname ""  \
                   [winfo class $target]
           incr vTcl(Tree_cnt)
           append vTcl(py_initfunc) \
             "$vTcl(tab2)self.T$vTcl(Tree_cnt) = "
           append vTcl(py_initfunc) \
                           "self.$widname.subwidget_list\['hlist'\]\n"
           # Here we need the special attribute
           vTcl:python_load_cmd $target "T$vTcl(Tree_cnt)"
           append vTcl(py_initfunc) \
                   "$vTcl(tab2)self.$widname.autosetmode()\n"
       }
       TixNoteBook {
           append vTcl(py_initfunc) \
                   "$vTcl(tab2)self.$widname = Tix.NoteBook($pframe)\n"
           append vTcl(py_initfunc) \
                   "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
           vTcl:python_configure_widget $target $widname ""  \
                   [winfo class $target]
           # Lets try to put out options for the nbframe subwidget
           set sub_name [join [concat $widname "_nbframe"] ""]
           append vTcl(py_initfunc) \
           "$vTcl(tab2)self.$sub_name = self.$widname.subwidget_list\[\"nbframe\"\]\n"
           set i [$target subwidget nbframe]
           vTcl:python_configure_widget \
                   $i $widname $ $widclass "self.$sub_name"

           # Lets try to add all the page subwidgets
           set p_c 0
           foreach p [$target pages] {
               set p_c [expr $p_c + 1]
               append vTcl(py_initfunc) \
               "$vTcl(tab2)self.$widname.add(\"$p\""
               set options [vTcl:python_nb_page_configure \
                       $p $target $widname "self.page$p_c"]
               append vTcl(py_initfunc) "$options)\n"
           }
           # Now lets try adding the subwidgets of every page.
           # I know it could be done in one loop.
           set p_c 0
           foreach p [$target pages] {
               set p_c [expr $p_c + 1]
               append vTcl(py_initfunc) \
               "$vTcl(tab2)self.page$p_c = "\
                    "self.$widname.subwidget_list\[\"$p\"\]\n"
               set i [$target subwidget $p]
            vTcl:python_source_subwidgets $i $i
           }
        } TixScrolledText {
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname = Tix.ScrolledText($pframe)\n"
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
            vTcl:python_configure_widget $target $widname ""  \
                    [winfo class $target]
            set sub_name [join [concat $widname "_text"] ""]
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$sub_name = "\
                    "self.$widname.subwidget_list\[\"text\"\]\n"
            # Here we need the special attributes.
            vTcl:python_load_cmd $target $sub_name
            set i [$target subwidget text]
            vTcl:python_configure_widget $i $widname $ \
                    $widclass "self.$sub_name"
        } TixMeter {
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname = Tix.Meter($pframe)\n"
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.$widname.place(in_=$pframe,x=$x,y=$y)\n"
            vTcl:python_configure_widget $target $widname ""  \
                    [winfo class $target]
        }
        Menu { # ignore }
        default {
           puts "Error in vTcl:python_source_subwidgets"
           puts "target = $target, widclass = $widclass"
       }
   }
   if {$vTcl(w,alias) != ""} {
       append vTcl(py_initfunc) \
                "$vTcl(tab2)self.alias = self.$sub_name\n"
   }
   append vTcl(py_initfunc) [vTcl:python_dump_bind $target $widname]
}

proc vTcl:python_get_inner_frame {target} {
    global vTcl
    set output ""
    set pieces [split $target .]
    set max 0
    for {set i 1} {$i <= [llength $pieces]} {incr i} {
        set can [lindex $pieces $i]
        if {[string first "tix" $can] != -1} {
            set max $i
        }
    }
    if {$max > 2} {
        append f_name [lindex $pieces 2] "_" [lindex $pieces $max]
    } else {
        set f_name [lindex $pieces 2]
    }
     return f_name
}

proc vTcl:find_attribute {widget attribute} {
    global vTcl
    set config [$widget configure]
    foreach c  $config {
        if {[lindex $c 0] == $attribute} {
            return [lindex $c 4]
        }
    }
}

proc vTcl:python_process_me {command widget widname} {
    global vTcl
    set i [string first "%me" $vTcl(w,opt,$command,$widget)]
    set x $vTcl(w,opt,$command,$widget)
    if {$i > -1} {
        set head [string range $x 0  [expr $i - 1]]
        set tail [string range $x [expr $i + 3] end]
        set x $head
        append x "self.$widname"
        append x $tail
    }
    return $x
}

proc vTcl:python_load_cmd  {widget widname} {
    global vTcl
    if {[info exists vTcl(w,opt,-loadcommand,$widget)]} {
        set x [vTcl:python_process_me "-loadcommand" $widget $widname]
        append vTcl(py_initfunc) "$vTcl(tab2)$x # ldcmd $widget\n"
    }
}

proc vTcl:python_load_n_bind_cmds {widget widname} {
    global vTcl
    vTcl:python_load_cmd  $widget $widname
    if {[info exists vTcl(w,opt,-bindcommand,$widget)]} {
        set x [vTcl:python_process_me "-bindcommand" $widget $widname]
        append vTcl(py_initfunc) \
      "$vTcl(tab2)self.$widname.bind('<Double-1>',$x) # bindcmd $widget\n"
    }
}

proc vTcl:python_delete_leading_tabs {proc} {
    set p $proc
    set spaces ""
    while {[string first "\n" $p] == 0} {
        set p [string range $p 1 end]
        set spaces 1
    }
    while {[string first " " $p] == 0} {
        set p [string range $p 1 end]
        set spaces 1
    }
    while {[string first "\t" $p] == 0} {
        set p [string range $p 1 end]
        set spaces 1
    }
    if {$spaces == ""} {
        return $proc
    }
    return $p
}

proc vTcl:python_indent_lines {proc} {
    global vTcl
global log
    set p $proc
    set rem  ""
    set tail $proc
    set lines ""
    while {[string first "\n" $tail] > -1} {
        set i [string first "\n" $tail]
        set head [string range $tail 0 $i]
        set tail [string range $tail [expr $i + 1] end]
        append rem $vTcl(tab) $head
        set lines 1
    }
    if {$lines == ""} {
        return proc
    }
    return $rem
}

proc vTcl:python_subst_tabs {proc} {
    global vTcl
    set p $proc
    set res ""
    set tabs ""
    while {[string first "\t" $p] > -1} {
        set i [string first "\t" $p]
        set head [string range $p 0 $i]
        set tail [string range $p [expr $i + 1] end]
        append p $head $vTcl(tab) $tail
        set tabs 1
    }
    if {$tabs == ""} {
        return $proc
    }
    return $p
}

proc vTcl:python_print_proc {proc} {
    set p $proc
    set i [string first "\t" $p]
    #puts "proc:"
    puts $p
}

proc vTcl:python_gui_startup {geom classname} {
    global vTcl
    set start_gui "def vp_start_gui():\n"
    append start_gui "$vTcl(tab)global w\n"
    append start_gui "$vTcl(tab)root.title('$classname')\n"
    append start_gui "$vTcl(tab)root.geometry('$geom')\n"
    append start_gui "$vTcl(tab)w = $classname (root)\n"
    append start_gui "$vTcl(tab)init()\n"
    append start_gui "$vTcl(tab)root.mainloop()\n"
    return $start_gui
}

proc  vTcl:get_subwidget_options {widget hlist} {
    # This Proc puts options of the subwidget into the -options
    # string of the widget creation.  I think that I could have
    #used this in more places.
    global vTcl
    set w [$widget subwidget $hlist]
    set config [$w config]
    # The folowing loop was borrowed from vTcl:python_configure_widget
    # It needs to be modified to return not a configure call but a
    # list for -options.
    set options ""
    foreach c  $config {
        if {[llength $c] == 5 &&
            [string first "Cmd" [lindex $c 1]] ==  -1 &&
            [string first "command" [lindex $c 1]] ==  -1 &&
            [string first "Command" [lindex $c 1]] ==  -1 } {
            if {[lindex $c 3] != [lindex $c 4]} {
                set str [lindex $c 0]
                # get rid of the - sign.
                set len [string length $str]
                set sub [string range $str 1 [expr $len -1]]
                append options \
              "\\\n$vTcl(tab2)$vTcl(tab2)$hlist.[lindex $c 1] \"[lindex $c 4]\""
            }
        }
    }
    return $options
}

proc vTcl:python_nb_page_configure {p widget widname name} {
    # Configures the option of the pageframe widget
    global vTcl
    set output ""
    set config [$widget pageconfigure $p]
    foreach c  $config {
        if {[llength $c] == 5 && [string first "cmd" [lindex $c 1]] ==  -1 } {
            if {[lindex $c 3] != [lindex $c 4]} {
                set str [lindex $c 0]
                # get rid of the - sign.
                set len [string length $str]
                set sub [string range $str 1 [expr $len -1]]
                append output \
                     ",$sub=\"[lindex $c 4]\""
            }
        }
    }
    return $output
}

proc vTcl:create_balloon_code {window} {
    # Generates a code string containing any of the balloon
    # code needed.
    global vTcl
    set balloon_widgets [vTcl:find_widgets_with_balloon_msg $window]
    set balloon_code ""
    if {[llength $balloon_widgets] >= 0} {
        append balloon_code \
           "$vTcl(tab2)self.balloon = Tix.Balloon(master)\n"
        foreach w $balloon_widgets {
            set widname [vTcl:widget_2_widname $w]
            append balloon_code \
         "$vTcl(tab2)self.balloon.bind_widget(self.$widname,"\
         "balloonmsg=\"$vTcl($w,balloon_msg)\")\n"
        }
        append balloon_code \
           "$vTcl(tab2)self.balloon.configure(bg='blue', borderwidth=3)"
    }
    return $balloon_code
}

proc vTcl:widget_2_widname {widget} {
    #    set widname [vTcl:widget_2_widname $widget]
    set pieces [split $widget .]
    set no [llength $pieces]
    set first [lindex $pieces 2]
    set sublist [lrange $pieces 3 end]
    set widname $first
    # What I am doing is risky. NEEDS WORK
    foreach p $sublist {
        append widname "_" $p
    }
#    if {$no > 3} {
#        append widname "_" [lindex $pieces end]
#    }
    return $widname
}

proc vTcl:python_set_alias_object {target type sub_name} {
    global vTcl widget
    set sub_t [$target subwidget $type]
    if {[catch {set alias $widget(rev,$sub_t)}]} {
        return
    } else {
        append vTcl(py_initfunc) \
                "$vTcl(tab2)self.$alias = self.$sub_name\n"
    }
}

proc vTcl:python_alias {alias target} {
    # Update the current value of alias when the
    # alias field in the attribute editor is changed.
    global vTcl widget
    catch {
        unset widget($was)
        unset widget(rev,$target)
    }
    if {$alias != ""} {
        set widget($alias) $target
        set widget(rev,$target) $alias
    }
}

proc vTcl:python_dump_bind {target widname} {
    # This is a take off from vTcl:dump_widget_bind
    global vTcl
    set result ""
    if {[catch {bindtags $target \{$vTcl(bindtags,$target)\}}]} {
        return ""
    }
    set bindlist [lsort [bind $target]]
    foreach i $bindlist {
        set command [bind $target $i]
        set command [string trim $command]

        if {"$vTcl(bind,ignore)" == "" ||
               ![regexp "^($vTcl(bind,ignore))" $command] } {
            if { ![regexp "focus" $command] } { # This if is another of
                                                # those tix hacks.
                append result \
                        "$vTcl(tab2)self.$widname.bind('$i',$command)\n"
            }
        }
    }
    bindtags $target vTcl(b)
    return $result
}

proc x_compare {a b} {
    set alast [lindex $a end]
    set blast [lindex $b end]
    if {$alast <= $blast} {
        return -1
    } else {
        return 1
    }
}

proc vTcl:sort_menubuttons {} {
    global vTcl
    set sort_list {}
    foreach i $vTcl(menubutton_list) {
        set x [winfo x $i]
        set item [list $i $x]
        lappend sort_list $item
    }
    set new [lsort -command x_compare $sort_list]
    return $new
}

proc  vTcl:py_process_menubutton_list {} {
    global vTcl
    if {[llength vTcl(menubutton_list)] == 0} {
        return
    }
    # First step: sort the buttons by x-position.
    set m_list [vTcl:sort_menubuttons]
    set suffix 0
    foreach i $m_list {
        set target [lindex $i 0]
        set suffix [expr $suffix + 1]
        set suffix_name "m$suffix"
        #set suffix_name "m"   ;#  NEEDS WORK  experimenting.
        set widname [vTcl:widget_2_widname $target]
        if {! $vTcl(py_has_menus) } { # Initializes the menubar, just once
            append vTcl(py_initfunc) \
                    "\n$vTcl(tab2)self.menubar = Frame(master,"
            append vTcl(py_initfunc) \
                    "relief=RAISED, bd=2)\n"
            append vTcl(py_initfunc) \
                    "$vTcl(tab2)self.menubar.pack(side=TOP,fill=X)\n"
            set vTcl(py_has_menus) 1
        }
        append vTcl(py_initfunc) \
          "$vTcl(tab2)self.$widname = Menubutton(self.menubar, underline = 0)\n"
        set label [$target cget -text]
        if {$label == "Help"} {
            set pack_side RIGHT
        } else {
            set pack_side LEFT
        }
        append vTcl(py_initfunc) \
                "$vTcl(tab2)self.$widname.pack(side=$pack_side)\n"
        append vTcl(py_initfunc) \
                "$vTcl(tab2)self.$widname.configure(text=\"$label\")\n"
        # Here is where I generate the menu itself.
        append vTcl(py_initfunc) \
                "$vTcl(tab2)self.$widname\_$suffix_name = Menu(self.$widname)\n"
        vTcl:python_parse_menu $target $widname $suffix ;# analyzes the menu
        append vTcl(py_initfunc) \
          "$vTcl(tab2)self.$widname\[\'menu\'\] = self.$widname\_$suffix_name\n"
     }
}
# For debugging purposes.
proc printenv { args } {
    global env
    set maxl 0
    if {[llength $args] == 0} {
        set args [lsort [array names env]]
    }
    foreach x $args {
        if {[string length $x] > $maxl} {
            set maxl [string length $x]
        }
    }
    incr maxl 2
    foreach x $args {
        puts stdout [format "%*s = %s" $maxl $x $env($x)]
    }
}

