#!/usr/local/bin/python

import commands, os.path, sys
from distutils.core import setup, Extension
from distutils.sysconfig import *
from distutils.errors import *

def get_R_HOME():
    stat, rhome = commands.getstatusoutput('R RHOME')
    if stat:
        raise DistutilsExecError("couldn't execute the R interpreter.\n"
                                 "Check whether R is in the execution path.")
    return rhome

# to avoid strict prototypes errors from R includes
get_config_vars()['OPT'] = '-DNDEBUG -g -O3 -Wall'

# get the Python version
if sys.version[:3] == '2.2':
    DEFINE = []
    UNDEF = ['PRE_2_2']
else:
    DEFINE = [('PRE_2_2', None)]
    UNDEF = []

# configure the R paths
RHOME = get_R_HOME()
DEFINE.append(('R_HOME', '"%s"' %RHOME))
r_include = os.path.join(RHOME, 'include')
r_libs = os.path.join(RHOME, 'bin')

# check whether Numeric is present
try:
    import Numeric
    DEFINE.append(('WITH_NUMERIC', None))
except ImportError:
    UNDEF.append('WITH_NUMERIC')

# get the RPy version
from rpy_version import rpy_version

LONG_DESC = """RPy provides a robust Python interface to the R
programming language.  It can manage all kinds of R objects and can
execute arbitrary R functions. All the errors from the R language are
converted to Python exceptions."""

setup(name="rpy",
      version=rpy_version,
      description="Python interface to the R language",
      author="Walter Moreira",
      author_email="walterm@cmat.edu.uy",
      url="http://rpy.sourceforge.net",
      license="GPL",
      long_description=LONG_DESC,
      py_modules=['rpy', 'io', 'rpy_version'],
      ext_modules=[Extension("_rpymodule", ["src/rpymodule.c", "src/R_eval.c",
                                            "src/io.c", "src/setenv.c"],
                             include_dirs=[r_include, "src"],
                             libraries=["R"],
                             library_dirs=[r_libs],
                             define_macros=DEFINE,
                             undef_macros=UNDEF,
                             extra_compile_args=["-shared"])]
      )
