/******************************************** 
	copyright 2001 McMillan Enterprises, Inc.
	www.mcmillan-inc.com
*********************************************/
#if !defined(PWENGINE_H_INCLUDED_)
#define PWENGINE_H_INCLUDED_

#include "PWOBase.h"
#include "PWOMapping.h"
#include "PWOMSequence.h"
#ifdef WIN32
#include <windows.h>	// for GetCurrentThreadId
#else
#include <pthread.h>    // for pthread_self
#endif


/****************************************
	FreeThreadedBlock

  {
	FreeThreadedBlock codeblock;
	// do stuff that doesn't need Python lock
  }
*****************************************/

class FreeThreadedBlock
{
protected:
	PyThreadState *tstate;
public:
	FreeThreadedBlock() {tstate = PyEval_SaveThread();};
	virtual ~FreeThreadedBlock() {PyEval_RestoreThread(tstate);};
};

/****************************************
	PythonThreadedBlock

  // We've been called by someone else, and need to do some Python
  {
	PythonThreadedBlock codeblock;
	// do stuff that needs Python lock
  }
*****************************************/

class PythonThreadedBlock
{
protected:
	PyThreadState *tstate;
public:
	PythonThreadedBlock();
	PythonThreadedBlock(PyThreadState *tstate_);
	virtual ~PythonThreadedBlock();
};
#ifdef WIN32
#define THREADIDTYPE DWORD
#else
#define THREADIDTYPE pthread_t
#endif
class PWEngine
{
protected:
	static THREADIDTYPE *ids;
	static PyThreadState **threads; 
	static PyInterpreterState **interps; 
	static int size_;
	static int refcounter_;
public:
	PWEngine() {
		if (!ids) {
			size_ = 5;
			ids = new THREADIDTYPE[size_];
			threads = (PyThreadState**)new PyThreadState[size_];
			interps = (PyInterpreterState**)new PyInterpreterState[size_];
			for (int i=0; i < size_; ++i) {
				ids[i] = 0;
				threads[i] = 0;
				interps[i] = 0;
			}
		}
		++refcounter_;
	};
	~PWEngine() {
		if (--refcounter_ == 0) {
			delete [] ids;
			delete [] threads;
			delete [] interps;
			ids = 0;
			threads = 0;
			interps = 0;
		}
	};
	void recordThread(); // use this in initXXX to get a valid tstate & interp
	void recordThread(PyThreadState *tstate_);
	PyThreadState *getTstate();
};

#endif

