// PWOBase.cpp: implementation of the PWOBase class.
//
//////////////////////////////////////////////////////////////////////

#include "PWOBase.h"
#include "PWOSequence.h"
#include "PWOMSequence.h"
#include "PWONumber.h"
#include "PWOMapping.h"
#include "Python.h"
#include "PWOModule.h"
#include "PWOCallable.h"
#include "PWEngine.h"


static PyObject *test_list(PyObject *_self, PyObject *_args)
{
	try {
#if 1
		PWOBase args(_args);
		PWOList list(2);
		list[0] = PWONumber(1);
		PWONumber n1(1.0);
		list.setItem(1, n1);
		list.append(PWONumber((unsigned long)1));
		list.insert(0, args);
		PWOString rslt;
		PWOTuple fargs(list);
		rslt = PWOString::format((const char *)"you sent me '%s'\n and I added %d, %f, %ld\n",
			fargs);
		return rslt.disOwn();
#else
		PWOBase args(_args); // no leak
		PWOList list(2);	// no leak
		//PWONumber x(1); no leak, no net increase in refcount on 1
		list[0] = PWONumber(1);	// no leak, not increase in refcount on 1
		list.setItem(1, PWONumber(1.0)); // no leak
		list.insert(0, args);	// no leak
		//return PWOBase(Py_None).disOwn();
		return PWOTuple(list).disOwn();
#endif
	}
	catch (PWException e) {
		return e.toPython();
	}

}
static PyObject *test_number(PyObject *_self, PyObject *_args)
{
	try {
		PWOSequence args(_args);
		PWONumber x(args[0]);
		PWONumber y(args[1]);
		PWOList list(7);
		list[0] = (x + y);
		list[1] = (x - y);
		list[2] = (x * y);
		list[3] = (x / y);
		list[4] = (y % PWONumber(7));
		list[5] = (x >> y);
		list[6] = (x << y);
		PWOString rslt;
		PWOTuple fargs(list);
		rslt = PWOString::format((const char *)" + %f, - %f, * %f, / %f, %% %d, >> %f, << %f\n",
			fargs);
		return rslt.disOwn();
	}
	catch (PWException e) {
		return e.toPython();
	}
}
static PyObject *test_dict(PyObject *self, PyObject *_args)
{
	try {
		char key1[] = {"key1\0"};
		char key2[] = {"key2"};
		PWOString cr("\n");
		PWOMapping d;
		if (_args) {
			PWOBase args(_args);
			d[key1] = args;
		}
		else {
			d[key1] = PWONone().disOwn();
		}
		d[key2] = PWOBase(Py_None);
		d[PWONumber(1)] = PWOString("Hi there");
		d.delItem(key2);
		PWOString s("pi");
		d[(PyObject *)s] = PWONumber(3.141592);
		PWOString rslt(d.repr());
		rslt = rslt + d.items().repr() +
			cr + d.keys().repr() + 
			cr + d.values().repr();
		return rslt.disOwn();
	}
	catch (PWException e) {
		return e.toPython();
	}
}

static PyObject *test_modules(PyObject *self, PyObject *_args)
{
	try {
		PWOModule string(PyImport_ImportModule("string"));
		PWOMapping stringdict(string.getDict());
		PWOCallable sjoin(stringdict["join"]);
		PWOList words(4);
		words[0] = PWOString("this");
		words[1] = PWOString("is");
		words[2] = PWOString("a");
		words[3] = PWOString("test");
		PWOTuple args(1);
		args.setItem(0, words);
		PWOString rslt = sjoin.call(args);
		return rslt.disOwn();
	}
	catch (PWException e) {
		return e;
	}
}

static PyObject *test_threads(PyObject *self, PyObject *_args)
{
	try {
		PWEngine().recordThread();
		PWOString rslt("Didn't work");
		{
			FreeThreadedBlock outer;
			{
				PythonThreadedBlock inner;
				PWOModule string(PyImport_ImportModule("string"));
				PWOMapping stringdict(string.getDict());
				PWOCallable sjoin(stringdict["join"]);
				PWOList words(4);
				words[0] = PWOString("this");
				words[1] = PWOString("is");
				words[2] = PWOString("a");
				words[3] = PWOString("test");
				PWOTuple args(1);
				args.setItem(0, words);
				rslt = sjoin.call(args);
			}	// end of inner's scope. Should return to free threaded
		} // end of free threaded scope.
		return rslt.disOwn();
	}
	catch (PWException e) {
		return e;
	}
}

static PyMethodDef frobit_methods[] = {
	{"testlists",	test_list},
	{"testnumbers",	test_number},
	{"testdicts", test_dict},
	{"testmodules", test_modules},
	{"testthreads", test_threads},
	{NULL,		NULL}		/* sentinel */
};

extern "C"
#ifdef WIN32
void __declspec(dllexport)
#else
void
#endif
initfrobit()
{
	static PWEngine engine;
	engine.recordThread();
	Py_InitModule("frobit", frobit_methods);
}

/* old test code:

  PWOBase-----
	PWOBase test(args);
	PWOBase test2;
	test2 = test;
	bool equal = (test2.cmp(test) == 0);
	return PWOBase(Py_None).disOwn();
  PWONumber-----
	PWONumber x(_args);
	PWONumber y(args);
	PWONumber z;
	z = x + y;
	long a = (long) x;
	int b = (int) y;
	//return z.disOwn();
	return (x+x).disOwn();
  PWOSequence------
	PWOSequence s(_args);
	PWOSequence s2 = s + s;
	PWOBase item0 = s[0];
	int cnt = s2.count(item0);
	for (int i=0; i < s.len(); i++) 
		ok = s[i] == s2[i];
	bool ok = s2.delItem(0);
	PWOSequence s3 = s2.getSlice(s.len()/2, s.len());
	s2.delSlice(s.len()/2, s.len()-1);
	return (s2 + s3).disOwn();
*/
