/******************************************** 
	copyright 1999 McMillan Enterprises, Inc.
	www.mcmillan-inc.com
*********************************************/
#include "PWOSequence.h"
#include "PWOMSequence.h"
#include "PWOMapping.h"
#include "PWOCallable.h"
#include "PWEngine.h"
#include "PWONumber.h"

	// incref new owner, and decref old owner, and adjust to new owner
void PWOBase::GrabRef(PyObject* newObj)
{
		// be careful to incref before decref if old is same as new
	Py_XINCREF(newObj);
	Py_XDECREF(_own);
	_own = _obj = newObj;
}

PWOTuple::PWOTuple(const PWOList& list) : PWOSequence (PyList_AsTuple(list)) { LoseRef(_obj); }

PWOListMmbr::PWOListMmbr(PyObject* obj, PWOList& parent, int ndx) 
	: PWOBase(obj), _parent(parent), _ndx(ndx) {
}

PWOListMmbr& PWOListMmbr::operator=(const PWOBase& other) {
	GrabRef(other);
	//Py_XINCREF(_obj);	// this one is for setItem to steal
	_parent.setItem(_ndx, *this);
	return *this;
}
PWOMappingMmbr& PWOMappingMmbr::operator=(const PWOBase& other) {
	GrabRef(other);
	_parent.setItem(_key, *this);
	return *this;
}
PWOBase PWOCallable::call() const {
	static PWOTuple _empty;
	PyObject *rslt = PyEval_CallObjectWithKeywords(*this, _empty, NULL);
	if (rslt == 0)
		throw PWCallbackException();
	return LoseRef(PWOBase(rslt));
}
PWOBase PWOCallable::call(PWOTuple& args) const {
	PyObject *rslt = PyEval_CallObjectWithKeywords(*this, args, NULL);
	if (rslt == 0)
		throw PWCallbackException();
	return LoseRef(PWOBase(rslt));
}
PWOBase PWOCallable::call(PWOTuple& args, PWOMapping& kws) const {
	PyObject *rslt = PyEval_CallObjectWithKeywords(*this, args, kws);
	if (rslt == 0)
		throw PWCallbackException();
	return LoseRef(PWOBase(rslt));
}
PWCallbackException::PWCallbackException() {
	PyObject *val, *tb;
	PyErr_Fetch(&py_exception, &val, &tb);
	Py_XDECREF(tb);
	Py_XDECREF(val);
}
THREADIDTYPE *PWEngine::ids;
PyThreadState **PWEngine::threads = 0;
PyInterpreterState **PWEngine::interps = 0;
int PWEngine::refcounter_ = 0;
int PWEngine::size_ = 0;
void PWEngine::recordThread() {
	PyThreadState *thisthread = PyThreadState_Swap(NULL);
	PyThreadState_Swap(thisthread);
	recordThread(thisthread);
}
void PWEngine::recordThread(PyThreadState *tstate_) {
#ifdef WIN32
	THREADIDTYPE osid = GetCurrentThreadId();
#else
	THREADIDTYPE osid = pthread_self();
#endif
	int i = 0;
	for (; i < size_ && ids[i]; ++i) {
		if (ids[i] == osid) {
			threads[i] = tstate_;
			//fprintf(stderr, "PWEngine.recordThread found osid %d\n", osid);
			return;
		}
	}
	if (i == size_) {
		//fprintf(stderr, "PWEngine resizing...\n");
		THREADIDTYPE *oldids = ids;
		PyThreadState **oldthreads = threads;
		PyInterpreterState **oldinterps = interps;
		int oldsize = size_;
		size_ += 5;
		ids = new THREADIDTYPE[size_];
		threads = (PyThreadState**)new PyThreadState[size_];
		interps = (PyInterpreterState**)new PyInterpreterState[size_];
		int j;
		for (j=0; j < oldsize; ++j) {
			ids[j] = oldids[j];
			threads[j] = oldthreads[j];
			interps[j] = oldinterps[j];
		}
		for (; j < size_; ++j) {
			ids[j] = 0;
			threads[j] = 0;
			interps[j] = 0;
		}
		delete [] oldids;
		delete [] oldthreads;
		delete [] oldinterps;
	}
	ids[i] = osid;
	threads[i] = tstate_;
	interps[i] = tstate_->interp;
	//fprintf(stderr, "PWEngine.recordThread set osid %d -> tstate %x\n", osid, tstate_);
}
PyThreadState *PWEngine::getTstate() {
#ifdef WIN32
	THREADIDTYPE osid = GetCurrentThreadId();
#else
	THREADIDTYPE osid = pthread_self();
#endif
	for (int i=0; i<size_; ++i) {
		if (ids[i] == osid) {
			//fprintf(stderr, "PWEngine.getTstate found osid %d -> %x\n", osid, threads[i]);
			return threads[i];
		}
	}
	if (interps[0])
		return PyThreadState_New(interps[0]);
	throw PWException(PyExc_RuntimeError, "Cannot acquire a Python thread");
	return 0;
}
PythonThreadedBlock::PythonThreadedBlock() {
	tstate = PWEngine().getTstate();
	PyEval_RestoreThread(tstate);
}
PythonThreadedBlock::PythonThreadedBlock(PyThreadState *tstate_) {
	PyEval_RestoreThread(tstate_);
}
PythonThreadedBlock::~PythonThreadedBlock() {
	PyEval_SaveThread();
}
